# Copyright 2013 VMware, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
from neutron_lib import constants as n_consts
from neutron_lib import context
from neutron_lib import exceptions as n_exc
from oslo_config import cfg

from neutron.tests import base
from neutron.tests.unit import testlib_api

from vmware_nsx.api_client import exception
from vmware_nsx.common import exceptions as p_exc
from vmware_nsx.db import lsn_db
from vmware_nsx.dhcp_meta import constants
from vmware_nsx.dhcp_meta import lsnmanager as lsn_man
from vmware_nsx.dhcp_meta import migration as mig_man
from vmware_nsx.dhcp_meta import nsx
from vmware_nsx.dhcp_meta import rpc


class DhcpMetadataBuilderTestCase(base.BaseTestCase):

    def setUp(self):
        super(DhcpMetadataBuilderTestCase, self).setUp()
        self.builder = mig_man.DhcpMetadataBuilder(mock.Mock(), mock.Mock())
        self.network_id = 'foo_network_id'
        self.subnet_id = 'foo_subnet_id'
        self.router_id = 'foo_router_id'

    def test_dhcp_agent_get_all(self):
        expected = []
        self.builder.plugin.list_dhcp_agents_hosting_network.return_value = (
            {'agents': expected})
        agents = self.builder.dhcp_agent_get_all(mock.ANY, self.network_id)
        self.assertEqual(expected, agents)

    def test_dhcp_port_get_all(self):
        expected = []
        self.builder.plugin.get_ports.return_value = expected
        ports = self.builder.dhcp_port_get_all(mock.ANY, self.network_id)
        self.assertEqual(expected, ports)

    def test_router_id_get(self):
        port = {
            'device_id': self.router_id,
            'network_id': self.network_id,
            'fixed_ips': [{'subnet_id': self.subnet_id}]
        }
        subnet = {
            'id': self.subnet_id,
            'network_id': self.network_id
        }
        self.builder.plugin.get_ports.return_value = [port]
        result = self.builder.router_id_get(context, subnet)
        self.assertEqual(self.router_id, result)

    def test_router_id_get_none_subnet(self):
        self.assertIsNone(self.builder.router_id_get(mock.ANY, None))

    def test_router_id_get_none_no_router(self):
        self.builder.plugin.get_ports.return_value = []
        subnet = {'network_id': self.network_id}
        self.assertIsNone(self.builder.router_id_get(mock.ANY, subnet))

    def test_metadata_deallocate(self):
        self.builder.metadata_deallocate(
            mock.ANY, self.router_id, self.subnet_id)
        self.assertTrue(self.builder.plugin.remove_router_interface.call_count)

    def test_metadata_allocate(self):
        self.builder.metadata_allocate(
            mock.ANY, self.router_id, self.subnet_id)
        self.assertTrue(self.builder.plugin.add_router_interface.call_count)

    def test_dhcp_deallocate(self):
        agents = [{'id': 'foo_agent_id'}]
        ports = [{'id': 'foo_port_id'}]
        self.builder.dhcp_deallocate(mock.ANY, self.network_id, agents, ports)
        self.assertTrue(
            self.builder.plugin.remove_network_from_dhcp_agent.call_count)
        self.assertTrue(self.builder.plugin.delete_port.call_count)

    def _test_dhcp_allocate(self, subnet, expected_notify_count):
        with mock.patch.object(mig_man.nsx, 'handle_network_dhcp_access') as f:
            self.builder.dhcp_allocate(mock.ANY, self.network_id, subnet)
            self.assertTrue(f.call_count)
            self.assertEqual(expected_notify_count,
                             self.builder.notifier.notify.call_count)

    def test_dhcp_allocate(self):
        subnet = {'network_id': self.network_id, 'id': self.subnet_id}
        self._test_dhcp_allocate(subnet, 2)

    def test_dhcp_allocate_none_subnet(self):
        self._test_dhcp_allocate(None, 0)


class MigrationManagerTestCase(base.BaseTestCase):

    def setUp(self):
        super(MigrationManagerTestCase, self).setUp()
        self.manager = mig_man.MigrationManager(mock.Mock(),
                                                mock.Mock(),
                                                mock.Mock())
        self.network_id = 'foo_network_id'
        self.router_id = 'foo_router_id'
        self.subnet_id = 'foo_subnet_id'
        self.mock_builder_p = mock.patch.object(self.manager, 'builder')
        self.mock_builder = self.mock_builder_p.start()

    def _test_validate(self, lsn_exists=False, ext_net=False, subnets=None):
        network = {'router:external': ext_net}
        self.manager.manager.lsn_exists.return_value = lsn_exists
        self.manager.plugin.get_network.return_value = network
        self.manager.plugin.get_subnets.return_value = subnets
        result = self.manager.validate(mock.ANY, self.network_id)
        if len(subnets):
            self.assertEqual(subnets[0], result)
        else:
            self.assertIsNone(result)

    def test_validate_no_subnets(self):
        self._test_validate(subnets=[])

    def test_validate_with_one_subnet(self):
        self._test_validate(subnets=[{'cidr': '0.0.0.0/0'}])

    def test_validate_raise_conflict_many_subnets(self):
        self.assertRaises(p_exc.LsnMigrationConflict,
                          self._test_validate,
                          subnets=[{'id': 'sub1'}, {'id': 'sub2'}])

    def test_validate_raise_conflict_lsn_exists(self):
        self.assertRaises(p_exc.LsnMigrationConflict,
                          self._test_validate,
                          lsn_exists=True)

    def test_validate_raise_badrequest_external_net(self):
        self.assertRaises(n_exc.BadRequest,
                          self._test_validate,
                          ext_net=True)

    def test_validate_raise_badrequest_metadata_net(self):
        self.assertRaises(n_exc.BadRequest,
                          self._test_validate,
                          ext_net=False,
                          subnets=[{'cidr': rpc.METADATA_SUBNET_CIDR}])

    def _test_migrate(self, router, subnet, expected_calls):
        self.mock_builder.router_id_get.return_value = router
        self.manager.migrate(mock.ANY, self.network_id, subnet)
        # testing the exact the order of calls is important
        self.assertEqual(expected_calls, self.mock_builder.mock_calls)

    def test_migrate(self):
        subnet = {
            'id': self.subnet_id,
            'network_id': self.network_id
        }
        call_sequence = [
            mock.call.router_id_get(mock.ANY, subnet),
            mock.call.metadata_deallocate(
                mock.ANY, self.router_id, self.subnet_id),
            mock.call.dhcp_agent_get_all(mock.ANY, self.network_id),
            mock.call.dhcp_port_get_all(mock.ANY, self.network_id),
            mock.call.dhcp_deallocate(
                mock.ANY, self.network_id, mock.ANY, mock.ANY),
            mock.call.dhcp_allocate(mock.ANY, self.network_id, subnet),
            mock.call.metadata_allocate(
                mock.ANY, self.router_id, self.subnet_id)
        ]
        self._test_migrate(self.router_id, subnet, call_sequence)

    def test_migrate_no_router_uplink(self):
        subnet = {
            'id': self.subnet_id,
            'network_id': self.network_id
        }
        call_sequence = [
            mock.call.router_id_get(mock.ANY, subnet),
            mock.call.dhcp_agent_get_all(mock.ANY, self.network_id),
            mock.call.dhcp_port_get_all(mock.ANY, self.network_id),
            mock.call.dhcp_deallocate(
                mock.ANY, self.network_id, mock.ANY, mock.ANY),
            mock.call.dhcp_allocate(mock.ANY, self.network_id, subnet),
        ]
        self._test_migrate(None, subnet, call_sequence)

    def test_migrate_no_subnet(self):
        call_sequence = [
            mock.call.router_id_get(mock.ANY, None),
            mock.call.dhcp_allocate(mock.ANY, self.network_id, None),
        ]
        self._test_migrate(None, None, call_sequence)

    def _test_report(self, lsn_attrs, expected):
        self.manager.manager.lsn_port_get.return_value = lsn_attrs
        report = self.manager.report(mock.ANY, self.network_id, self.subnet_id)
        self.assertEqual(expected, report)

    def test_report_for_lsn(self):
        self._test_report(('foo_lsn_id', 'foo_lsn_port_id'),
                          {'ports': ['foo_lsn_port_id'],
                           'services': ['foo_lsn_id'], 'type': 'lsn'})

    def test_report_for_lsn_without_lsn_port(self):
        self._test_report(('foo_lsn_id', None),
                          {'ports': [],
                           'services': ['foo_lsn_id'], 'type': 'lsn'})

    def _test_report_for_lsn_without_subnet(self, validated_subnet):
        with mock.patch.object(self.manager.plugin, 'get_subnets',
                               return_value=validated_subnet):
            self.manager.manager.lsn_port_get.return_value = (
                ('foo_lsn_id', 'foo_lsn_port_id'))
            report = self.manager.report(context, self.network_id)
            expected = {
                'ports': ['foo_lsn_port_id'] if validated_subnet else [],
                'services': ['foo_lsn_id'], 'type': 'lsn'
            }
            self.assertEqual(expected, report)

    def test_report_for_lsn_without_subnet_subnet_found(self):
        self._test_report_for_lsn_without_subnet([{'id': self.subnet_id}])

    def test_report_for_lsn_without_subnet_subnet_not_found(self):
        self.manager.manager.lsn_get.return_value = 'foo_lsn_id'
        self._test_report_for_lsn_without_subnet(None)

    def test_report_for_dhcp_agent(self):
        self.manager.manager.lsn_port_get.return_value = (None, None)
        self.mock_builder.dhcp_agent_get_all.return_value = (
            [{'id': 'foo_agent_id'}])
        self.mock_builder.dhcp_port_get_all.return_value = (
            [{'id': 'foo_dhcp_port_id'}])
        result = self.manager.report(mock.ANY, self.network_id, self.subnet_id)
        expected = {
            'ports': ['foo_dhcp_port_id'],
            'services': ['foo_agent_id'],
            'type': 'agent'
        }
        self.assertEqual(expected, result)


class LsnManagerTestCase(base.BaseTestCase):

    def setUp(self):
        super(LsnManagerTestCase, self).setUp()
        self.net_id = 'foo_network_id'
        self.sub_id = 'foo_subnet_id'
        self.port_id = 'foo_port_id'
        self.lsn_id = 'foo_lsn_id'
        self.mac = 'aa:bb:cc:dd:ee:ff'
        self.switch_id = 'foo_switch_id'
        self.lsn_port_id = 'foo_lsn_port_id'
        self.tenant_id = 'foo_tenant_id'
        self.manager = lsn_man.LsnManager(mock.Mock())
        self.context = context.get_admin_context()
        self.mock_lsn_api_p = mock.patch.object(lsn_man, 'lsn_api')
        self.mock_lsn_api = self.mock_lsn_api_p.start()
        self.mock_nsx_utils_p = mock.patch.object(lsn_man, 'nsx_utils')
        self.mock_nsx_utils = self.mock_nsx_utils_p.start()
        nsx.register_dhcp_opts(cfg)
        nsx.register_metadata_opts(cfg)

    def test_lsn_get(self):
        self.mock_lsn_api.lsn_for_network_get.return_value = self.lsn_id
        expected = self.manager.lsn_get(mock.ANY, self.net_id)
        self.mock_lsn_api.lsn_for_network_get.assert_called_once_with(
            mock.ANY, self.net_id)
        self.assertEqual(expected, self.lsn_id)

    def _test_lsn_get_raise_not_found_with_exc(self, exc):
        self.mock_lsn_api.lsn_for_network_get.side_effect = exc
        self.assertRaises(p_exc.LsnNotFound,
                          self.manager.lsn_get,
                          mock.ANY, self.net_id)
        self.mock_lsn_api.lsn_for_network_get.assert_called_once_with(
            mock.ANY, self.net_id)

    def test_lsn_get_raise_not_found_with_not_found(self):
        self._test_lsn_get_raise_not_found_with_exc(n_exc.NotFound)

    def test_lsn_get_raise_not_found_with_api_error(self):
        self._test_lsn_get_raise_not_found_with_exc(exception.NsxApiException)

    def _test_lsn_get_silent_raise_with_exc(self, exc):
        self.mock_lsn_api.lsn_for_network_get.side_effect = exc
        expected = self.manager.lsn_get(
            mock.ANY, self.net_id, raise_on_err=False)
        self.mock_lsn_api.lsn_for_network_get.assert_called_once_with(
            mock.ANY, self.net_id)
        self.assertIsNone(expected)

    def test_lsn_get_silent_raise_with_not_found(self):
        self._test_lsn_get_silent_raise_with_exc(n_exc.NotFound)

    def test_lsn_get_silent_raise_with_api_error(self):
        self._test_lsn_get_silent_raise_with_exc(exception.NsxApiException)

    def test_lsn_create(self):
        self.mock_lsn_api.lsn_for_network_create.return_value = self.lsn_id
        self.manager.lsn_create(mock.ANY, self.net_id)
        self.mock_lsn_api.lsn_for_network_create.assert_called_once_with(
            mock.ANY, self.net_id)

    def test_lsn_create_raise_api_error(self):
        self.mock_lsn_api.lsn_for_network_create.side_effect = (
            exception.NsxApiException)
        self.assertRaises(p_exc.NsxPluginException,
                          self.manager.lsn_create,
                          mock.ANY, self.net_id)
        self.mock_lsn_api.lsn_for_network_create.assert_called_once_with(
            mock.ANY, self.net_id)

    def test_lsn_delete(self):
        self.manager.lsn_delete(mock.ANY, self.lsn_id)
        self.mock_lsn_api.lsn_delete.assert_called_once_with(
            mock.ANY, self.lsn_id)

    def _test_lsn_delete_with_exc(self, exc):
        self.mock_lsn_api.lsn_delete.side_effect = exc
        self.manager.lsn_delete(mock.ANY, self.lsn_id)
        self.mock_lsn_api.lsn_delete.assert_called_once_with(
            mock.ANY, self.lsn_id)

    def test_lsn_delete_with_not_found(self):
        self._test_lsn_delete_with_exc(n_exc.NotFound)

    def test_lsn_delete_api_exception(self):
        self._test_lsn_delete_with_exc(exception.NsxApiException)

    def test_lsn_delete_by_network(self):
        self.mock_lsn_api.lsn_for_network_get.return_value = self.lsn_id
        with mock.patch.object(self.manager, 'lsn_delete') as f:
            self.manager.lsn_delete_by_network(mock.ANY, self.net_id)
            self.mock_lsn_api.lsn_for_network_get.assert_called_once_with(
                mock.ANY, self.net_id)
            f.assert_called_once_with(mock.ANY, self.lsn_id)

    def _test_lsn_delete_by_network_with_exc(self, exc):
        self.mock_lsn_api.lsn_for_network_get.side_effect = exc
        with mock.patch.object(lsn_man.LOG, 'warning') as l:
            self.manager.lsn_delete_by_network(mock.ANY, self.net_id)
            self.assertEqual(1, l.call_count)

    def test_lsn_delete_by_network_with_not_found(self):
        self._test_lsn_delete_by_network_with_exc(n_exc.NotFound)

    def test_lsn_delete_by_network_with_not_api_error(self):
        self._test_lsn_delete_by_network_with_exc(exception.NsxApiException)

    def test_lsn_port_get(self):
        self.mock_lsn_api.lsn_port_by_subnet_get.return_value = (
            self.lsn_port_id)
        with mock.patch.object(
            self.manager, 'lsn_get', return_value=self.lsn_id):
            expected = self.manager.lsn_port_get(
                mock.ANY, self.net_id, self.sub_id)
            self.assertEqual(expected, (self.lsn_id, self.lsn_port_id))

    def test_lsn_port_get_lsn_not_found_on_raise(self):
        with mock.patch.object(
            self.manager, 'lsn_get',
            side_effect=p_exc.LsnNotFound(entity='network',
                                          entity_id=self.net_id)):
            self.assertRaises(p_exc.LsnNotFound,
                              self.manager.lsn_port_get,
                              mock.ANY, self.net_id, self.sub_id)

    def test_lsn_port_get_lsn_not_found_silent_raise(self):
        with mock.patch.object(self.manager, 'lsn_get', return_value=None):
            result = self.manager.lsn_port_get(
                mock.ANY, self.net_id, self.sub_id, raise_on_err=False)
            expected = (None, None)
            self.assertEqual(expected, result)

    def test_lsn_port_get_port_not_found_on_raise(self):
        self.mock_lsn_api.lsn_port_by_subnet_get.side_effect = n_exc.NotFound
        with mock.patch.object(
            self.manager, 'lsn_get', return_value=self.lsn_id):
            self.assertRaises(p_exc.LsnPortNotFound,
                              self.manager.lsn_port_get,
                              mock.ANY, self.net_id, self.sub_id)

    def test_lsn_port_get_port_not_found_silent_raise(self):
        self.mock_lsn_api.lsn_port_by_subnet_get.side_effect = n_exc.NotFound
        with mock.patch.object(
            self.manager, 'lsn_get', return_value=self.lsn_id):
            result = self.manager.lsn_port_get(
                mock.ANY, self.net_id, self.sub_id, raise_on_err=False)
            expected = (self.lsn_id, None)
            self.assertEqual(expected, result)

    def test_lsn_port_create(self):
        self.mock_lsn_api.lsn_port_create.return_value = self.lsn_port_id
        expected = self.manager.lsn_port_create(mock.ANY, mock.ANY, mock.ANY)
        self.assertEqual(expected, self.lsn_port_id)

    def _test_lsn_port_create_with_exc(self, exc, expected):
        self.mock_lsn_api.lsn_port_create.side_effect = exc
        self.assertRaises(expected,
                          self.manager.lsn_port_create,
                          mock.ANY, mock.ANY, mock.ANY)

    def test_lsn_port_create_with_not_found(self):
        self._test_lsn_port_create_with_exc(n_exc.NotFound, p_exc.LsnNotFound)

    def test_lsn_port_create_api_exception(self):
        self._test_lsn_port_create_with_exc(exception.NsxApiException,
                                            p_exc.NsxPluginException)

    def test_lsn_port_delete(self):
        self.manager.lsn_port_delete(mock.ANY, mock.ANY, mock.ANY)
        self.assertEqual(1, self.mock_lsn_api.lsn_port_delete.call_count)

    def _test_lsn_port_delete_with_exc(self, exc):
        self.mock_lsn_api.lsn_port_delete.side_effect = exc
        with mock.patch.object(lsn_man.LOG, 'warning') as l:
            self.manager.lsn_port_delete(mock.ANY, mock.ANY, mock.ANY)
            self.assertEqual(1, self.mock_lsn_api.lsn_port_delete.call_count)
            self.assertEqual(1, l.call_count)

    def test_lsn_port_delete_with_not_found(self):
        self._test_lsn_port_delete_with_exc(n_exc.NotFound)

    def test_lsn_port_delete_api_exception(self):
        self._test_lsn_port_delete_with_exc(exception.NsxApiException)

    def _test_lsn_port_dhcp_setup(self, ret_val, sub):
        self.mock_nsx_utils.get_nsx_switch_ids.return_value = [self.switch_id]
        self.mock_lsn_api.lsn_port_create.return_value = self.lsn_port_id
        with mock.patch.object(
            self.manager, 'lsn_get', return_value=self.lsn_id):
            with mock.patch.object(lsn_man.switch_api,
                                   'get_port_by_neutron_tag'):
                expected = self.manager.lsn_port_dhcp_setup(
                    mock.Mock(), mock.ANY, mock.ANY,
                    mock.ANY, subnet_config=sub)
                self.assertEqual(
                    1, self.mock_lsn_api.lsn_port_create.call_count)
                self.assertEqual(
                    1, self.mock_lsn_api.lsn_port_plug_network.call_count)
                self.assertEqual(expected, ret_val)

    def test_lsn_port_dhcp_setup(self):
        self._test_lsn_port_dhcp_setup((self.lsn_id, self.lsn_port_id), None)

    def test_lsn_port_dhcp_setup_with_config(self):
        with mock.patch.object(self.manager, 'lsn_port_dhcp_configure') as f:
            self._test_lsn_port_dhcp_setup(None, mock.ANY)
            self.assertEqual(1, f.call_count)

    def test_lsn_port_dhcp_setup_with_not_found(self):
        self.mock_nsx_utils.get_nsx_switch_ids.return_value = [self.switch_id]
        with mock.patch.object(lsn_man.switch_api,
                               'get_port_by_neutron_tag') as f:
            f.side_effect = n_exc.NotFound
            self.assertRaises(p_exc.PortConfigurationError,
                              self.manager.lsn_port_dhcp_setup,
                              mock.Mock(), mock.ANY, mock.ANY, mock.ANY)

    def test_lsn_port_dhcp_setup_with_conflict(self):
        self.mock_lsn_api.lsn_port_plug_network.side_effect = (
            p_exc.LsnConfigurationConflict(lsn_id=self.lsn_id))
        self.mock_nsx_utils.get_nsx_switch_ids.return_value = [self.switch_id]
        with mock.patch.object(lsn_man.switch_api, 'get_port_by_neutron_tag'):
            with mock.patch.object(self.manager, 'lsn_port_delete') as g:
                self.assertRaises(p_exc.PortConfigurationError,
                                  self.manager.lsn_port_dhcp_setup,
                                  mock.Mock(), mock.ANY, mock.ANY, mock.ANY)
                self.assertEqual(1, g.call_count)

    def _test_lsn_port_dhcp_configure_with_subnet(
        self, expected, dns=None, gw=None, routes=None):
        subnet = {
            'enable_dhcp': True,
            'dns_nameservers': dns or [],
            'gateway_ip': gw,
            'host_routes': routes
        }
        self.manager.lsn_port_dhcp_configure(mock.ANY, self.lsn_id,
                                             self.lsn_port_id, subnet)
        self.mock_lsn_api.lsn_port_dhcp_configure.assert_called_once_with(
            mock.ANY, self.lsn_id, self.lsn_port_id, subnet['enable_dhcp'],
            expected)

    def test_lsn_port_dhcp_configure(self):
        expected = {
            'routers': '127.0.0.1',
            'default_lease_time': cfg.CONF.NSX_DHCP.default_lease_time,
            'domain_name': cfg.CONF.NSX_DHCP.domain_name
        }
        self._test_lsn_port_dhcp_configure_with_subnet(
            expected, dns=[], gw='127.0.0.1', routes=[])

    def test_lsn_port_dhcp_configure_gatewayless(self):
        expected = {
            'default_lease_time': cfg.CONF.NSX_DHCP.default_lease_time,
            'domain_name': cfg.CONF.NSX_DHCP.domain_name
        }
        self._test_lsn_port_dhcp_configure_with_subnet(expected, gw=None)

    def test_lsn_port_dhcp_configure_with_extra_dns_servers(self):
        expected = {
            'default_lease_time': cfg.CONF.NSX_DHCP.default_lease_time,
            'domain_name_servers': '8.8.8.8,9.9.9.9',
            'domain_name': cfg.CONF.NSX_DHCP.domain_name
        }
        self._test_lsn_port_dhcp_configure_with_subnet(
            expected, dns=['8.8.8.8', '9.9.9.9'])

    def test_lsn_port_dhcp_configure_with_host_routes(self):
        expected = {
            'default_lease_time': cfg.CONF.NSX_DHCP.default_lease_time,
            'domain_name': cfg.CONF.NSX_DHCP.domain_name,
            'classless_static_routes': '8.8.8.8,9.9.9.9'
        }
        self._test_lsn_port_dhcp_configure_with_subnet(
            expected, routes=['8.8.8.8', '9.9.9.9'])

    def _test_lsn_metadata_configure(self, is_enabled):
        with mock.patch.object(self.manager, 'lsn_port_dispose') as f:
            self.manager.plugin.get_subnet.return_value = (
                {'network_id': self.net_id})
            self.manager.lsn_metadata_configure(mock.ANY,
                                                self.sub_id, is_enabled)
            expected = {
                'metadata_server_port': 8775,
                'metadata_server_ip': '127.0.0.1',
                'metadata_proxy_shared_secret': ''
            }
            self.mock_lsn_api.lsn_metadata_configure.assert_called_once_with(
                mock.ANY, mock.ANY, is_enabled, expected)
            if is_enabled:
                self.assertEqual(
                    1, self.mock_lsn_api.lsn_port_by_subnet_get.call_count)
            else:
                self.assertEqual(1, f.call_count)

    def test_lsn_metadata_configure_enabled(self):
        self._test_lsn_metadata_configure(True)

    def test_lsn_metadata_configure_disabled(self):
        self._test_lsn_metadata_configure(False)

    def test_lsn_metadata_configure_not_found(self):
        self.mock_lsn_api.lsn_metadata_configure.side_effect = (
            p_exc.LsnNotFound(entity='lsn', entity_id=self.lsn_id))
        self.manager.plugin.get_subnet.return_value = (
            {'network_id': self.net_id})
        self.assertRaises(p_exc.NsxPluginException,
                          self.manager.lsn_metadata_configure,
                          mock.ANY, self.sub_id, True)

    def test_lsn_port_metadata_setup(self):
        subnet = {
            'cidr': '0.0.0.0/0',
            'id': self.sub_id,
            'network_id': self.net_id,
            'tenant_id': self.tenant_id
        }
        expected_data = {
            'subnet_id': subnet['id'],
            'ip_address': subnet['cidr'],
            'mac_address': constants.METADATA_MAC
        }
        self.mock_nsx_utils.get_nsx_switch_ids.return_value = [self.switch_id]
        with mock.patch.object(lsn_man.switch_api, 'create_lport') as f:
            with mock.patch.object(self.manager, 'lsn_port_create') as g:
                f.return_value = {'uuid': self.port_id}
                self.manager.lsn_port_metadata_setup(
                    self.context, self.lsn_id, subnet)
                (self.mock_lsn_api.lsn_port_plug_network.
                 assert_called_once_with(mock.ANY, self.lsn_id,
                                         mock.ANY, self.port_id))
                g.assert_called_once_with(
                    self.context, self.lsn_id, expected_data)

    def test_lsn_port_metadata_setup_raise_not_found(self):
        subnet = {
            'cidr': '0.0.0.0/0',
            'id': self.sub_id,
            'network_id': self.net_id,
            'tenant_id': self.tenant_id
        }
        self.mock_nsx_utils.get_nsx_switch_ids.return_value = [self.switch_id]
        with mock.patch.object(lsn_man.switch_api, 'create_lport') as f:
            f.side_effect = n_exc.NotFound
            self.assertRaises(p_exc.PortConfigurationError,
                              self.manager.lsn_port_metadata_setup,
                              mock.Mock(), self.lsn_id, subnet)

    def test_lsn_port_metadata_setup_raise_conflict(self):
        subnet = {
            'cidr': '0.0.0.0/0',
            'id': self.sub_id,
            'network_id': self.net_id,
            'tenant_id': self.tenant_id
        }
        self.mock_nsx_utils.get_nsx_switch_ids.return_value = [self.switch_id]
        with mock.patch.object(lsn_man.switch_api, 'create_lport') as f:
            with mock.patch.object(lsn_man.switch_api, 'delete_port') as g:
                f.return_value = {'uuid': self.port_id}
                self.mock_lsn_api.lsn_port_plug_network.side_effect = (
                    p_exc.LsnConfigurationConflict(lsn_id=self.lsn_id))
                self.assertRaises(p_exc.PortConfigurationError,
                                  self.manager.lsn_port_metadata_setup,
                                  mock.Mock(), self.lsn_id, subnet)
                self.assertEqual(1,
                                 self.mock_lsn_api.lsn_port_delete.call_count)
                self.assertEqual(1, g.call_count)

    def _test_lsn_port_dispose_with_values(self, lsn_id, lsn_port_id, count):
        with mock.patch.object(self.manager,
                               'lsn_port_get_by_mac',
                               return_value=(lsn_id, lsn_port_id)):
            self.manager.lsn_port_dispose(mock.ANY, self.net_id, self.mac)
            self.assertEqual(count,
                             self.mock_lsn_api.lsn_port_delete.call_count)

    def test_lsn_port_dispose(self):
        self._test_lsn_port_dispose_with_values(
            self.lsn_id, self.lsn_port_id, 1)

    def test_lsn_port_dispose_meta_mac(self):
        self.mac = constants.METADATA_MAC
        with mock.patch.object(lsn_man.switch_api,
                               'get_port_by_neutron_tag') as f:
            with mock.patch.object(lsn_man.switch_api, 'delete_port') as g:
                f.return_value = {'uuid': self.port_id}
                self._test_lsn_port_dispose_with_values(
                    self.lsn_id, self.lsn_port_id, 1)
                f.assert_called_once_with(
                    mock.ANY, self.net_id, constants.METADATA_PORT_ID)
                g.assert_called_once_with(mock.ANY, self.net_id, self.port_id)

    def test_lsn_port_dispose_lsn_not_found(self):
        self._test_lsn_port_dispose_with_values(None, None, 0)

    def test_lsn_port_dispose_lsn_port_not_found(self):
        self._test_lsn_port_dispose_with_values(self.lsn_id, None, 0)

    def test_lsn_port_dispose_api_error(self):
        self.mock_lsn_api.lsn_port_delete.side_effect = (
            exception.NsxApiException)
        with mock.patch.object(lsn_man.LOG, 'warning') as l:
            self.manager.lsn_port_dispose(mock.ANY, self.net_id, self.mac)
            self.assertEqual(1, l.call_count)

    def test_lsn_port_host_conf(self):
        with mock.patch.object(self.manager,
                               'lsn_port_get',
                               return_value=(self.lsn_id, self.lsn_port_id)):
            f = mock.Mock()
            self.manager._lsn_port_host_conf(mock.ANY, self.net_id,
                                             self.sub_id, mock.ANY, f)
            self.assertEqual(1, f.call_count)

    def test_lsn_port_host_conf_lsn_port_not_found(self):
        with mock.patch.object(
            self.manager, 'lsn_port_get', return_value=(None, None)) as f:
            self.manager._lsn_port_host_conf(
                mock.ANY, self.net_id, self.sub_id, mock.ANY, mock.Mock())
            self.assertEqual(1, f.call_count)

    def _test_lsn_port_update(self, dhcp=None, meta=None):
        self.manager.lsn_port_update(
            mock.ANY, self.net_id, self.sub_id, dhcp, meta)
        count = 1 if dhcp else 0
        count = count + 1 if meta else count
        self.assertEqual(count, (self.mock_lsn_api.
                                 lsn_port_host_entries_update.call_count))

    def test_lsn_port_update(self):
        self._test_lsn_port_update()

    def test_lsn_port_update_dhcp_meta(self):
        self._test_lsn_port_update(mock.ANY, mock.ANY)

    def test_lsn_port_update_dhcp_and_nometa(self):
        self._test_lsn_port_update(mock.ANY, None)

    def test_lsn_port_update_nodhcp_and_nmeta(self):
        self._test_lsn_port_update(None, mock.ANY)

    def test_lsn_port_update_raise_error(self):
        self.mock_lsn_api.lsn_port_host_entries_update.side_effect = (
            exception.NsxApiException)
        self.assertRaises(p_exc.PortConfigurationError,
                          self.manager.lsn_port_update,
                          mock.ANY, mock.ANY, mock.ANY, mock.ANY)


class PersistentLsnManagerTestCase(testlib_api.SqlTestCase):

    def setUp(self):
        super(PersistentLsnManagerTestCase, self).setUp()
        self.net_id = 'foo_network_id'
        self.sub_id = 'foo_subnet_id'
        self.port_id = 'foo_port_id'
        self.lsn_id = 'foo_lsn_id'
        self.mac = 'aa:bb:cc:dd:ee:ff'
        self.lsn_port_id = 'foo_lsn_port_id'
        self.tenant_id = 'foo_tenant_id'
        nsx.register_dhcp_opts(cfg)
        nsx.register_metadata_opts(cfg)
        lsn_man.register_lsn_opts(cfg)
        self.manager = lsn_man.PersistentLsnManager(mock.Mock())
        self.context = context.get_admin_context()
        self.mock_lsn_api_p = mock.patch.object(lsn_man, 'lsn_api')
        self.mock_lsn_api = self.mock_lsn_api_p.start()

    def test_lsn_get(self):
        lsn_db.lsn_add(self.context, self.net_id, self.lsn_id)
        result = self.manager.lsn_get(self.context, self.net_id)
        self.assertEqual(self.lsn_id, result)

    def test_lsn_get_raise_not_found(self):
        self.assertRaises(p_exc.LsnNotFound,
                          self.manager.lsn_get, self.context, self.net_id)

    def test_lsn_get_silent_not_found(self):
        result = self.manager.lsn_get(
            self.context, self.net_id, raise_on_err=False)
        self.assertIsNone(result)

    def test_lsn_get_sync_on_missing(self):
        cfg.CONF.set_override('sync_on_missing_data', True, 'NSX_LSN')
        self.manager = lsn_man.PersistentLsnManager(mock.Mock())
        with mock.patch.object(self.manager, 'lsn_save') as f:
            self.manager.lsn_get(self.context, self.net_id, raise_on_err=True)
            self.assertTrue(self.mock_lsn_api.lsn_for_network_get.call_count)
            self.assertTrue(f.call_count)

    def test_lsn_save(self):
        self.manager.lsn_save(self.context, self.net_id, self.lsn_id)
        result = self.manager.lsn_get(self.context, self.net_id)
        self.assertEqual(self.lsn_id, result)

    def test_lsn_create(self):
        self.mock_lsn_api.lsn_for_network_create.return_value = self.lsn_id
        with mock.patch.object(self.manager, 'lsn_save') as f:
            result = self.manager.lsn_create(self.context, self.net_id)
            self.assertTrue(
                self.mock_lsn_api.lsn_for_network_create.call_count)
            self.assertTrue(f.call_count)
            self.assertEqual(self.lsn_id, result)

    def test_lsn_create_failure(self):
        with mock.patch.object(
            self.manager, 'lsn_save',
            side_effect=p_exc.NsxPluginException(err_msg='')):
            self.assertRaises(p_exc.NsxPluginException,
                              self.manager.lsn_create,
                              self.context, self.net_id)
            self.assertTrue(self.mock_lsn_api.lsn_delete.call_count)

    def test_lsn_delete(self):
        self.mock_lsn_api.lsn_for_network_create.return_value = self.lsn_id
        self.manager.lsn_create(self.context, self.net_id)
        self.manager.lsn_delete(self.context, self.lsn_id)
        self.assertIsNone(self.manager.lsn_get(
            self.context, self.net_id, raise_on_err=False))

    def test_lsn_delete_not_existent(self):
        self.manager.lsn_delete(self.context, self.lsn_id)
        self.assertTrue(self.mock_lsn_api.lsn_delete.call_count)

    def test_lsn_port_get(self):
        lsn_db.lsn_add(self.context, self.net_id, self.lsn_id)
        lsn_db.lsn_port_add_for_lsn(self.context, self.lsn_port_id,
                                    self.sub_id, self.mac, self.lsn_id)
        res = self.manager.lsn_port_get(self.context, self.net_id, self.sub_id)
        self.assertEqual((self.lsn_id, self.lsn_port_id), res)

    def test_lsn_port_get_raise_not_found(self):
        self.assertRaises(p_exc.LsnPortNotFound,
                          self.manager.lsn_port_get,
                          self.context, self.net_id, self.sub_id)

    def test_lsn_port_get_silent_not_found(self):
        result = self.manager.lsn_port_get(
            self.context, self.net_id, self.sub_id, raise_on_err=False)
        expected = (None, None)
        self.assertEqual(expected, result)

    def test_lsn_port_get_sync_on_missing(self):
        return
        cfg.CONF.set_override('sync_on_missing_data', True, 'NSX_LSN')
        self.manager = lsn_man.PersistentLsnManager(mock.Mock())
        self.mock_lsn_api.lsn_for_network_get.return_value = self.lsn_id
        self.mock_lsn_api.lsn_port_by_subnet_get.return_value = (
            self.lsn_id, self.lsn_port_id)
        with mock.patch.object(self.manager, 'lsn_save') as f:
            with mock.patch.object(self.manager, 'lsn_port_save') as g:
                self.manager.lsn_port_get(
                    self.context, self.net_id, self.sub_id)
                self.assertTrue(
                    self.mock_lsn_api.lsn_port_by_subnet_get.call_count)
                self.assertTrue(
                    self.mock_lsn_api.lsn_port_info_get.call_count)
                self.assertTrue(f.call_count)
                self.assertTrue(g.call_count)

    def test_lsn_port_get_by_mac(self):
        lsn_db.lsn_add(self.context, self.net_id, self.lsn_id)
        lsn_db.lsn_port_add_for_lsn(self.context, self.lsn_port_id,
                                    self.sub_id, self.mac, self.lsn_id)
        res = self.manager.lsn_port_get_by_mac(
            self.context, self.net_id, self.mac)
        self.assertEqual((self.lsn_id, self.lsn_port_id), res)

    def test_lsn_port_get_by_mac_raise_not_found(self):
        self.assertRaises(p_exc.LsnPortNotFound,
                          self.manager.lsn_port_get_by_mac,
                          self.context, self.net_id, self.sub_id)

    def test_lsn_port_get_by_mac_silent_not_found(self):
        result = self.manager.lsn_port_get_by_mac(
            self.context, self.net_id, self.sub_id, raise_on_err=False)
        expected = (None, None)
        self.assertEqual(expected, result)

    def test_lsn_port_create(self):
        lsn_db.lsn_add(self.context, self.net_id, self.lsn_id)
        self.mock_lsn_api.lsn_port_create.return_value = self.lsn_port_id
        subnet = {'subnet_id': self.sub_id, 'mac_address': self.mac}
        with mock.patch.object(self.manager, 'lsn_port_save') as f:
            result = self.manager.lsn_port_create(
                self.context, self.net_id, subnet)
            self.assertTrue(
                self.mock_lsn_api.lsn_port_create.call_count)
            self.assertTrue(f.call_count)
            self.assertEqual(self.lsn_port_id, result)

    def test_lsn_port_create_failure(self):
        subnet = {'subnet_id': self.sub_id, 'mac_address': self.mac}
        with mock.patch.object(
            self.manager, 'lsn_port_save',
            side_effect=p_exc.NsxPluginException(err_msg='')):
            self.assertRaises(p_exc.NsxPluginException,
                              self.manager.lsn_port_create,
                              self.context, self.net_id, subnet)
            self.assertTrue(self.mock_lsn_api.lsn_port_delete.call_count)

    def test_lsn_port_delete(self):
        lsn_db.lsn_add(self.context, self.net_id, self.lsn_id)
        lsn_db.lsn_port_add_for_lsn(self.context, self.lsn_port_id,
                                    self.sub_id, self.mac, self.lsn_id)
        self.manager.lsn_port_delete(
            self.context, self.lsn_id, self.lsn_port_id)
        expected = (None, None)
        self.assertEqual(expected, self.manager.lsn_port_get(
            self.context, self.lsn_id, self.sub_id, raise_on_err=False))

    def test_lsn_port_delete_not_existent(self):
        self.manager.lsn_port_delete(
            self.context, self.lsn_id, self.lsn_port_id)
        self.assertTrue(self.mock_lsn_api.lsn_port_delete.call_count)

    def test_lsn_port_save(self):
        self.manager.lsn_save(self.context, self.net_id, self.lsn_id)
        self.manager.lsn_port_save(self.context, self.lsn_port_id,
                                   self.sub_id, self.mac, self.lsn_id)
        result = self.manager.lsn_port_get(
            self.context, self.net_id, self.sub_id, raise_on_err=False)
        self.assertEqual((self.lsn_id, self.lsn_port_id), result)


class DhcpAgentNotifyAPITestCase(base.BaseTestCase):

    def setUp(self):
        super(DhcpAgentNotifyAPITestCase, self).setUp()
        self.notifier = nsx.DhcpAgentNotifyAPI(mock.Mock(), mock.Mock())
        self.plugin = self.notifier.plugin
        self.lsn_manager = self.notifier.lsn_manager

    def _test_notify_port_update(
        self, ports, expected_count, expected_args=None):
        port = {
            'id': 'foo_port_id',
            'network_id': 'foo_network_id',
            'fixed_ips': [{'subnet_id': 'foo_subnet_id'}]
        }
        self.notifier.plugin.get_ports.return_value = ports
        self.notifier.notify(mock.ANY, {'port': port}, 'port.update.end')
        self.lsn_manager.lsn_port_update.assert_has_calls(expected_args)

    def test_notify_ports_update_no_ports(self):
        self._test_notify_port_update(None, 0, [])
        self._test_notify_port_update([], 0, [])

    def test_notify_ports_update_one_port(self):
        ports = [{
            'fixed_ips': [{'subnet_id': 'foo_subnet_id',
                           'ip_address': '1.2.3.4'}],
            'device_id': 'foo_device_id',
            'device_owner': 'foo_device_owner',
            'mac_address': 'fa:16:3e:da:1d:46'
        }]
        call_args = mock.call(
            mock.ANY, 'foo_network_id', 'foo_subnet_id',
            dhcp=[{'ip_address': '1.2.3.4',
                   'mac_address': 'fa:16:3e:da:1d:46'}],
            meta=[{'instance_id': 'foo_device_id',
                   'ip_address': '1.2.3.4'}])
        self._test_notify_port_update(ports, 1, [call_args])

    def test_notify_ports_update_ports_with_empty_device_id(self):
        ports = [{
            'fixed_ips': [{'subnet_id': 'foo_subnet_id',
                           'ip_address': '1.2.3.4'}],
            'device_id': '',
            'device_owner': 'foo_device_owner',
            'mac_address': 'fa:16:3e:da:1d:46'
        }]
        call_args = mock.call(
            mock.ANY, 'foo_network_id', 'foo_subnet_id',
            dhcp=[{'ip_address': '1.2.3.4',
                   'mac_address': 'fa:16:3e:da:1d:46'}],
            meta=[]
        )
        self._test_notify_port_update(ports, 1, [call_args])

    def test_notify_ports_update_ports_with_no_fixed_ips(self):
        ports = [{
            'fixed_ips': [],
            'device_id': 'foo_device_id',
            'device_owner': 'foo_device_owner',
            'mac_address': 'fa:16:3e:da:1d:46'
        }]
        call_args = mock.call(
            mock.ANY, 'foo_network_id', 'foo_subnet_id', dhcp=[], meta=[])
        self._test_notify_port_update(ports, 1, [call_args])

    def test_notify_ports_update_ports_with_no_fixed_ips_and_no_device(self):
        ports = [{
            'fixed_ips': [],
            'device_id': '',
            'device_owner': 'foo_device_owner',
            'mac_address': 'fa:16:3e:da:1d:46'
        }]
        call_args = mock.call(
            mock.ANY, 'foo_network_id', 'foo_subnet_id', dhcp=[], meta=[])
        self._test_notify_port_update(ports, 0, [call_args])

    def test_notify_ports_update_with_special_ports(self):
        ports = [{'fixed_ips': [],
                  'device_id': '',
                  'device_owner': n_consts.DEVICE_OWNER_DHCP,
                  'mac_address': 'fa:16:3e:da:1d:46'},
                 {'fixed_ips': [{'subnet_id': 'foo_subnet_id',
                                 'ip_address': '1.2.3.4'}],
                  'device_id': 'foo_device_id',
                  'device_owner': n_consts.DEVICE_OWNER_ROUTER_GW,
                  'mac_address': 'fa:16:3e:da:1d:46'}]
        call_args = mock.call(
            mock.ANY, 'foo_network_id', 'foo_subnet_id', dhcp=[], meta=[])
        self._test_notify_port_update(ports, 0, [call_args])

    def test_notify_ports_update_many_ports(self):
        ports = [{'fixed_ips': [],
                  'device_id': '',
                  'device_owner': 'foo_device_owner',
                  'mac_address': 'fa:16:3e:da:1d:46'},
                 {'fixed_ips': [{'subnet_id': 'foo_subnet_id',
                                 'ip_address': '1.2.3.4'}],
                  'device_id': 'foo_device_id',
                  'device_owner': 'foo_device_owner',
                  'mac_address': 'fa:16:3e:da:1d:46'}]
        call_args = mock.call(
            mock.ANY, 'foo_network_id', 'foo_subnet_id',
            dhcp=[{'ip_address': '1.2.3.4',
                   'mac_address': 'fa:16:3e:da:1d:46'}],
            meta=[{'instance_id': 'foo_device_id',
                   'ip_address': '1.2.3.4'}])
        self._test_notify_port_update(ports, 1, [call_args])

    def _test_notify_subnet_action(self, action):
        with mock.patch.object(self.notifier, '_subnet_%s' % action) as f:
            self.notifier._handle_subnet_dhcp_access[action] = f
            subnet = {'subnet': mock.ANY}
            self.notifier.notify(
                mock.ANY, subnet, 'subnet.%s.end' % action)
            f.assert_called_once_with(mock.ANY, subnet)

    def test_notify_subnet_create(self):
        self._test_notify_subnet_action('create')

    def test_notify_subnet_update(self):
        self._test_notify_subnet_action('update')

    def test_notify_subnet_delete(self):
        self._test_notify_subnet_action('delete')

    def _test_subnet_create(self, enable_dhcp, exc=None,
                            exc_obj=None, call_notify=True):
        subnet = {
            'id': 'foo_subnet_id',
            'enable_dhcp': enable_dhcp,
            'network_id': 'foo_network_id',
            'tenant_id': 'foo_tenant_id',
            'cidr': '0.0.0.0/0'
        }
        if exc:
            self.plugin.create_port.side_effect = exc_obj or exc
            self.assertRaises(exc,
                              self.notifier.notify,
                              mock.ANY,
                              {'subnet': subnet},
                              'subnet.create.end')
            self.plugin.delete_subnet.assert_called_with(
                mock.ANY, subnet['id'])
        else:
            if call_notify:
                self.notifier.notify(
                    mock.ANY, {'subnet': subnet}, 'subnet.create.end')
            if enable_dhcp:
                dhcp_port = {
                    'name': '',
                    'admin_state_up': True,
                    'network_id': 'foo_network_id',
                    'tenant_id': 'foo_tenant_id',
                    'device_owner': n_consts.DEVICE_OWNER_DHCP,
                    'mac_address': mock.ANY,
                    'fixed_ips': [{'subnet_id': 'foo_subnet_id'}],
                    'device_id': ''
                }
                self.plugin.create_port.assert_called_once_with(
                    mock.ANY, {'port': dhcp_port})
            else:
                self.assertEqual(0, self.plugin.create_port.call_count)

    def test_subnet_create_enabled_dhcp(self):
        self._test_subnet_create(True)

    def test_subnet_create_disabled_dhcp(self):
        self._test_subnet_create(False)

    def test_subnet_create_raise_port_config_error(self):
        with mock.patch.object(nsx.db_base_plugin_v2.NeutronDbPluginV2,
                               'delete_port') as d:
            self._test_subnet_create(
                True,
                exc=n_exc.Conflict,
                exc_obj=p_exc.PortConfigurationError(lsn_id='foo_lsn_id',
                                                     net_id='foo_net_id',
                                                     port_id='foo_port_id'))
            d.assert_called_once_with(self.plugin, mock.ANY, 'foo_port_id')

    def test_subnet_update(self):
        subnet = {
            'id': 'foo_subnet_id',
            'network_id': 'foo_network_id',
        }
        self.lsn_manager.lsn_port_get.return_value = ('foo_lsn_id',
                                                      'foo_lsn_port_id')
        self.notifier.notify(
            mock.ANY, {'subnet': subnet}, 'subnet.update.end')
        self.lsn_manager.lsn_port_dhcp_configure.assert_called_once_with(
            mock.ANY, 'foo_lsn_id', 'foo_lsn_port_id', subnet)

    def test_subnet_update_raise_lsn_not_found(self):
        subnet = {
            'id': 'foo_subnet_id',
            'network_id': 'foo_network_id',
        }
        self.lsn_manager.lsn_port_get.side_effect = (
            p_exc.LsnNotFound(entity='network',
                              entity_id=subnet['network_id']))
        self.assertRaises(p_exc.LsnNotFound,
                          self.notifier.notify,
                          mock.ANY, {'subnet': subnet}, 'subnet.update.end')

    def _test_subnet_update_lsn_port_not_found(self, dhcp_port):
        subnet = {
            'id': 'foo_subnet_id',
            'enable_dhcp': True,
            'network_id': 'foo_network_id',
            'tenant_id': 'foo_tenant_id'
        }
        self.lsn_manager.lsn_port_get.side_effect = (
            p_exc.LsnPortNotFound(lsn_id='foo_lsn_id',
                                  entity='subnet',
                                  entity_id=subnet['id']))
        self.notifier.plugin.get_ports.return_value = dhcp_port
        count = 0 if dhcp_port is None else 1
        with mock.patch.object(nsx, 'handle_port_dhcp_access') as h:
            self.notifier.notify(
                mock.ANY, {'subnet': subnet}, 'subnet.update.end')
            self.assertEqual(count, h.call_count)
            if not dhcp_port:
                self._test_subnet_create(enable_dhcp=True,
                                         exc=None, call_notify=False)

    def test_subnet_update_lsn_port_not_found_without_dhcp_port(self):
        self._test_subnet_update_lsn_port_not_found(None)

    def test_subnet_update_lsn_port_not_found_with_dhcp_port(self):
        self._test_subnet_update_lsn_port_not_found([mock.ANY])

    def _test_subnet_delete(self, ports=None):
        subnet = {
            'id': 'foo_subnet_id',
            'network_id': 'foo_network_id',
            'cidr': '0.0.0.0/0'
        }
        self.plugin.get_ports.return_value = ports
        self.notifier.notify(mock.ANY, {'subnet': subnet}, 'subnet.delete.end')
        filters = {
            'network_id': [subnet['network_id']],
            'device_owner': [n_consts.DEVICE_OWNER_DHCP]
        }
        self.plugin.get_ports.assert_called_once_with(
            mock.ANY, filters=filters)
        if ports:
            self.plugin.delete_port.assert_called_once_with(
                mock.ANY, ports[0]['id'])
        else:
            self.assertEqual(0, self.plugin.delete_port.call_count)

    def test_subnet_delete_enabled_dhcp_no_ports(self):
        self._test_subnet_delete()

    def test_subnet_delete_enabled_dhcp_with_dhcp_port(self):
        self._test_subnet_delete([{'id': 'foo_port_id'}])


class DhcpTestCase(base.BaseTestCase):

    def setUp(self):
        super(DhcpTestCase, self).setUp()
        self.plugin = mock.Mock()
        self.plugin.lsn_manager = mock.Mock()

    def test_handle_create_network(self):
        network = {'id': 'foo_network_id'}
        nsx.handle_network_dhcp_access(
            self.plugin, mock.ANY, network, 'create_network')
        self.plugin.lsn_manager.lsn_create.assert_called_once_with(
            mock.ANY, network['id'])

    def test_handle_create_network_router_external(self):
        network = {'id': 'foo_network_id', 'router:external': True}
        nsx.handle_network_dhcp_access(
            self.plugin, mock.ANY, network, 'create_network')
        self.assertFalse(self.plugin.lsn_manager.lsn_create.call_count)

    def test_handle_delete_network(self):
        network_id = 'foo_network_id'
        self.plugin.lsn_manager.lsn_delete_by_network.return_value = (
            'foo_lsn_id')
        nsx.handle_network_dhcp_access(
            self.plugin, mock.ANY, network_id, 'delete_network')
        self.plugin.lsn_manager.lsn_delete_by_network.assert_called_once_with(
            mock.ANY, 'foo_network_id')

    def _test_handle_create_dhcp_owner_port(self, exc=None):
        subnet = {
            'cidr': '0.0.0.0/0',
            'id': 'foo_subnet_id'
        }
        port = {
            'id': 'foo_port_id',
            'device_owner': n_consts.DEVICE_OWNER_DHCP,
            'mac_address': 'aa:bb:cc:dd:ee:ff',
            'network_id': 'foo_network_id',
            'fixed_ips': [{'subnet_id': subnet['id']}]
        }
        expected_data = {
            'subnet_id': subnet['id'],
            'ip_address': subnet['cidr'],
            'mac_address': port['mac_address']
        }
        self.plugin.get_subnet.return_value = subnet
        if exc is None:
            nsx.handle_port_dhcp_access(
                self.plugin, mock.ANY, port, 'create_port')
            (self.plugin.lsn_manager.lsn_port_dhcp_setup.
             assert_called_once_with(mock.ANY, port['network_id'],
                                     port['id'], expected_data, subnet))
        else:
            self.plugin.lsn_manager.lsn_port_dhcp_setup.side_effect = exc
            self.assertRaises(n_exc.NeutronException,
                              nsx.handle_port_dhcp_access,
                              self.plugin, mock.ANY, port, 'create_port')

    def test_handle_create_dhcp_owner_port(self):
        self._test_handle_create_dhcp_owner_port()

    def test_handle_create_dhcp_owner_port_raise_port_config_error(self):
        config_error = p_exc.PortConfigurationError(lsn_id='foo_lsn_id',
                                                    net_id='foo_net_id',
                                                    port_id='foo_port_id')
        self._test_handle_create_dhcp_owner_port(exc=config_error)

    def test_handle_delete_dhcp_owner_port(self):
        port = {
            'id': 'foo_port_id',
            'device_owner': n_consts.DEVICE_OWNER_DHCP,
            'network_id': 'foo_network_id',
            'fixed_ips': [],
            'mac_address': 'aa:bb:cc:dd:ee:ff'
        }
        nsx.handle_port_dhcp_access(self.plugin, mock.ANY, port, 'delete_port')
        self.plugin.lsn_manager.lsn_port_dispose.assert_called_once_with(
            mock.ANY, port['network_id'], port['mac_address'])

    def _test_handle_user_port(self, action, handler):
        port = {
            'id': 'foo_port_id',
            'device_owner': 'foo_device_owner',
            'network_id': 'foo_network_id',
            'mac_address': 'aa:bb:cc:dd:ee:ff',
            'fixed_ips': [{'subnet_id': 'foo_subnet_id',
                           'ip_address': '1.2.3.4'}]
        }
        expected_data = {
            'ip_address': '1.2.3.4',
            'mac_address': 'aa:bb:cc:dd:ee:ff'
        }
        self.plugin.get_subnet.return_value = {'enable_dhcp': True}
        nsx.handle_port_dhcp_access(self.plugin, mock.ANY, port, action)
        handler.assert_called_once_with(
            mock.ANY, port['network_id'], 'foo_subnet_id', expected_data)

    def test_handle_create_user_port(self):
        self._test_handle_user_port(
            'create_port', self.plugin.lsn_manager.lsn_port_dhcp_host_add)

    def test_handle_delete_user_port(self):
        self._test_handle_user_port(
            'delete_port', self.plugin.lsn_manager.lsn_port_dhcp_host_remove)

    def _test_handle_user_port_disabled_dhcp(self, action, handler):
        port = {
            'id': 'foo_port_id',
            'device_owner': 'foo_device_owner',
            'network_id': 'foo_network_id',
            'mac_address': 'aa:bb:cc:dd:ee:ff',
            'fixed_ips': [{'subnet_id': 'foo_subnet_id',
                           'ip_address': '1.2.3.4'}]
        }
        self.plugin.get_subnet.return_value = {'enable_dhcp': False}
        nsx.handle_port_dhcp_access(self.plugin, mock.ANY, port, action)
        self.assertEqual(0, handler.call_count)

    def test_handle_create_user_port_disabled_dhcp(self):
        self._test_handle_user_port_disabled_dhcp(
            'create_port', self.plugin.lsn_manager.lsn_port_dhcp_host_add)

    def test_handle_delete_user_port_disabled_dhcp(self):
        self._test_handle_user_port_disabled_dhcp(
            'delete_port', self.plugin.lsn_manager.lsn_port_dhcp_host_remove)

    def _test_handle_user_port_no_fixed_ips(self, action, handler):
        port = {
            'id': 'foo_port_id',
            'device_owner': 'foo_device_owner',
            'network_id': 'foo_network_id',
            'fixed_ips': []
        }
        nsx.handle_port_dhcp_access(self.plugin, mock.ANY, port, action)
        self.assertEqual(0, handler.call_count)

    def test_handle_create_user_port_no_fixed_ips(self):
        self._test_handle_user_port_no_fixed_ips(
            'create_port', self.plugin.lsn_manager.lsn_port_dhcp_host_add)

    def test_handle_delete_user_port_no_fixed_ips(self):
        self._test_handle_user_port_no_fixed_ips(
            'delete_port', self.plugin.lsn_manager.lsn_port_dhcp_host_remove)


class MetadataTestCase(base.BaseTestCase):

    def setUp(self):
        super(MetadataTestCase, self).setUp()
        self.plugin = mock.Mock()
        self.plugin.lsn_manager = mock.Mock()

    def _test_handle_port_metadata_access_special_owners(
        self, owner, dev_id='foo_device_id', ips=None):
        port = {
            'id': 'foo_port_id',
            'device_owner': owner,
            'device_id': dev_id,
            'fixed_ips': ips or []
        }
        nsx.handle_port_metadata_access(self.plugin, mock.ANY, port, mock.ANY)
        self.assertFalse(
            self.plugin.lsn_manager.lsn_port_meta_host_add.call_count)
        self.assertFalse(
            self.plugin.lsn_manager.lsn_port_meta_host_remove.call_count)

    def test_handle_port_metadata_access_external_network(self):
        port = {
            'id': 'foo_port_id',
            'device_owner': 'foo_device_owner',
            'device_id': 'foo_device_id',
            'network_id': 'foo_network_id',
            'fixed_ips': [{'subnet_id': 'foo_subnet'}]
        }
        self.plugin.get_network.return_value = {'router:external': True}
        nsx.handle_port_metadata_access(self.plugin, mock.ANY, port, mock.ANY)
        self.assertFalse(
            self.plugin.lsn_manager.lsn_port_meta_host_add.call_count)
        self.assertFalse(
            self.plugin.lsn_manager.lsn_port_meta_host_remove.call_count)

    def test_handle_port_metadata_access_dhcp_port(self):
        self._test_handle_port_metadata_access_special_owners(
            n_consts.DEVICE_OWNER_DHCP, [{'subnet_id': 'foo_subnet'}])

    def test_handle_port_metadata_access_router_port(self):
        self._test_handle_port_metadata_access_special_owners(
            n_consts.DEVICE_OWNER_ROUTER_INTF, [{'subnet_id': 'foo_subnet'}])

    def test_handle_port_metadata_access_no_device_id(self):
        self._test_handle_port_metadata_access_special_owners(
            n_consts.DEVICE_OWNER_DHCP, '')

    def test_handle_port_metadata_access_no_fixed_ips(self):
        self._test_handle_port_metadata_access_special_owners(
            'foo', 'foo', None)

    def _test_handle_port_metadata_access(self, is_delete, raise_exc=False):
        port = {
            'id': 'foo_port_id',
            'device_owner': 'foo_device_id',
            'network_id': 'foo_network_id',
            'device_id': 'foo_device_id',
            'tenant_id': 'foo_tenant_id',
            'fixed_ips': [
                {'subnet_id': 'foo_subnet_id', 'ip_address': '1.2.3.4'}
            ]
        }
        meta = {
            'instance_id': port['device_id'],
            'tenant_id': port['tenant_id'],
            'ip_address': port['fixed_ips'][0]['ip_address']
        }
        self.plugin.get_network.return_value = {'router:external': False}
        if is_delete:
            mock_func = self.plugin.lsn_manager.lsn_port_meta_host_remove
        else:
            mock_func = self.plugin.lsn_manager.lsn_port_meta_host_add
        if raise_exc:
            mock_func.side_effect = p_exc.PortConfigurationError(
                lsn_id='foo_lsn_id', net_id='foo_net_id', port_id=None)
            with mock.patch.object(nsx.db_base_plugin_v2.NeutronDbPluginV2,
                                   'delete_port') as d:
                self.assertRaises(p_exc.PortConfigurationError,
                                  nsx.handle_port_metadata_access,
                                  self.plugin, mock.ANY, port,
                                  is_delete=is_delete)
                if not is_delete:
                    d.assert_called_once_with(mock.ANY, mock.ANY, port['id'])
                else:
                    self.assertFalse(d.call_count)
        else:
            nsx.handle_port_metadata_access(
                self.plugin, mock.ANY, port, is_delete=is_delete)
        mock_func.assert_called_once_with(mock.ANY, mock.ANY, mock.ANY, meta)

    def test_handle_port_metadata_access_on_delete_true(self):
        self._test_handle_port_metadata_access(True)

    def test_handle_port_metadata_access_on_delete_false(self):
        self._test_handle_port_metadata_access(False)

    def test_handle_port_metadata_access_on_delete_true_raise(self):
        self._test_handle_port_metadata_access(True, raise_exc=True)

    def test_handle_port_metadata_access_on_delete_false_raise(self):
        self._test_handle_port_metadata_access(False, raise_exc=True)

    def _test_handle_router_metadata_access(
        self, is_port_found, raise_exc=False):
        subnet = {
            'id': 'foo_subnet_id',
            'network_id': 'foo_network_id'
        }
        interface = {
            'subnet_id': subnet['id'],
            'port_id': 'foo_port_id'
        }
        mock_func = self.plugin.lsn_manager.lsn_metadata_configure
        if not is_port_found:
            self.plugin.get_port.side_effect = n_exc.NotFound
        if raise_exc:
            with mock.patch.object(nsx.l3_db.L3_NAT_db_mixin,
                                   'remove_router_interface') as d:
                mock_func.side_effect = p_exc.NsxPluginException(err_msg='')
                self.assertRaises(p_exc.NsxPluginException,
                                  nsx.handle_router_metadata_access,
                                  self.plugin, mock.ANY, 'foo_router_id',
                                  interface)
                d.assert_called_once_with(mock.ANY, mock.ANY, 'foo_router_id',
                                          interface)
        else:
            nsx.handle_router_metadata_access(
                self.plugin, mock.ANY, 'foo_router_id', interface)
            mock_func.assert_called_once_with(
                mock.ANY, subnet['id'], is_port_found)

    def test_handle_router_metadata_access_add_interface(self):
        self._test_handle_router_metadata_access(True)

    def test_handle_router_metadata_access_delete_interface(self):
        self._test_handle_router_metadata_access(False)

    def test_handle_router_metadata_access_raise_error_on_add(self):
        self._test_handle_router_metadata_access(True, raise_exc=True)

    def test_handle_router_metadata_access_raise_error_on_delete(self):
        self._test_handle_router_metadata_access(True, raise_exc=False)



# Copyright 2013 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import converters
from neutron_lib.api import extensions
from neutron_lib import constants


MAC_LEARNING = 'mac_learning_enabled'
EXTENDED_ATTRIBUTES_2_0 = {
    'ports': {
        MAC_LEARNING: {'allow_post': True, 'allow_put': True,
                       'convert_to': converters.convert_to_boolean,
                       'default': constants.ATTR_NOT_SPECIFIED,
                       'is_visible': True},
    }
}


class Maclearning(extensions.ExtensionDescriptor):
    """Extension class supporting port mac learning."""

    @classmethod
    def get_name(cls):
        return "MAC Learning"

    @classmethod
    def get_alias(cls):
        return "mac-learning"

    @classmethod
    def get_description(cls):
        return "Provides MAC learning capabilities."

    @classmethod
    def get_updated(cls):
        return "2013-05-1T10:00:00-00:00"

    @classmethod
    def get_resources(cls):
        """Returns Ext Resources."""
        return []

    def get_extended_resources(self, version):
        if version == "2.0":
            return EXTENDED_ATTRIBUTES_2_0
        else:
            return {}



# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_log import log
import stevedore

from neutron_lib import exceptions as n_exc
from vmware_nsx.common import locking

LOG = log.getLogger(__name__)
ALL_DUMMY_JOB = {
    'name': 'all',
    'description': 'Execute all housekeepers',
    'enabled': True}


class NsxvHousekeeper(stevedore.named.NamedExtensionManager):
    def __init__(self, hk_ns, hk_jobs):
        self.readonly = cfg.CONF.nsxv.housekeeping_readonly
        if self.readonly:
            LOG.info('Housekeeper initialized in readonly mode')
        else:
            LOG.info('Housekeeper initialized')

        self.jobs = {}
        super(NsxvHousekeeper, self).__init__(
            hk_ns, hk_jobs, invoke_on_load=True, invoke_args=(self.readonly,))

        LOG.info("Loaded housekeeping job names: %s", self.names())
        for job in self:
            if job.obj.get_name() in cfg.CONF.nsxv.housekeeping_jobs:
                self.jobs[job.obj.get_name()] = job.obj

    def get(self, job_name):
        if job_name == ALL_DUMMY_JOB.get('name'):
            return ALL_DUMMY_JOB

        for job in self:
            name = job.obj.get_name()
            if job_name == name:
                return {'name': job_name,
                        'description': job.obj.get_description(),
                        'enabled': job_name in self.jobs}

        raise n_exc.ObjectNotFound(id=job_name)

    def list(self):
        results = [ALL_DUMMY_JOB]

        for job in self:
            job_name = job.obj.get_name()
            results.append({'name': job_name,
                            'description': job.obj.get_description(),
                            'enabled': job_name in self.jobs})

        return results

    def run(self, context, job_name):
        if context.is_admin:
            with locking.LockManager.get_lock('nsx-housekeeper'):
                if job_name == ALL_DUMMY_JOB.get('name'):
                    for job in self.jobs.values():
                        job.run(context)
                else:
                    job = self.jobs.get(job_name)
                    if job:
                        job.run(context)
                    else:
                        raise n_exc.ObjectNotFound(id=job_name)
        else:
            raise n_exc.AdminRequired()



# Copyright 2017 VMware, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import time

from oslo_config import cfg
from oslo_log import log as logging

from vmware_nsx.common import locking

LOG = logging.getLogger(__name__)


class EdgeDynamicRoutingDriver(object):

    """Edge driver API to implement the dynamic routing"""

    def __init__(self):
        # it will be initialized at subclass
        self.vcns = None
        self.ecmp_wait_time = cfg.CONF.nsxv.ecmp_wait_time

    def _prepare_bgp_config(self, bgp_config):
        bgp_config.setdefault('enabled', False)
        bgp_config.setdefault('bgpNeighbours', {'bgpNeighbours': []})
        bgp_config.setdefault('redistribution', {'rules': {'rules': []}})

        curr_neighbours = [{'bgpNeighbour': nbr} for nbr in
                           bgp_config['bgpNeighbours']['bgpNeighbours']]
        bgp_config['bgpNeighbours'] = curr_neighbours
        for nbr in curr_neighbours:
            bgp_filters = [{'bgpFilter': bf} for bf
                           in nbr['bgpNeighbour']['bgpFilters']['bgpFilters']]
            nbr['bgpNeighbour']['bgpFilters'] = bgp_filters
        redistribution_rules = [{'rule': rule} for rule in
                                bgp_config['redistribution']['rules']['rules']]
        bgp_config['redistribution']['rules'] = redistribution_rules

    def _get_routing_config(self, edge_id):
        h, config = self.vcns.get_edge_routing_config(edge_id)
        # Backend complains when adding this in the request.
        config.pop('featureType')
        config.pop('ospf')
        global_config = config['routingGlobalConfig']
        bgp_config = config.get('bgp', {})

        self._prepare_bgp_config(bgp_config)

        global_config.setdefault('ipPrefixes', {'ipPrefixes': []})
        curr_prefixes = [{'ipPrefix': prx}
                         for prx in global_config['ipPrefixes']['ipPrefixes']]
        global_config['ipPrefixes'] = curr_prefixes

        # Don't change any static routes.
        static_routing = config.get('staticRouting', {})
        static_routes = static_routing.get('staticRoutes', {})
        current_routes = [{'route': route}
                          for route in static_routes.get('staticRoutes', [])]
        static_routing['staticRoutes'] = current_routes
        return {'routing': config}

    def _update_routing_config(self, edge_id, **kwargs):
        routing_config = self._get_routing_config(edge_id)
        global_config = routing_config['routing']['routingGlobalConfig']
        current_prefixes = global_config['ipPrefixes']

        global_config['ecmp'] = True

        if 'router_id' in kwargs:
            global_config['routerId'] = kwargs['router_id']

        current_prefixes[:] = [p for p in current_prefixes
                               if p['ipPrefix']['name'] not in
                               kwargs.get('prefixes_to_remove', [])]
        # Avoid adding duplicate rules when shared router relocation
        current_prefixes.extend([p for p in kwargs.get('prefixes_to_add', [])
                                 if p not in current_prefixes])

        self.vcns.update_edge_routing_config(edge_id, routing_config)

    def _reset_routing_global_config(self, edge_id):
        routing_config = self._get_routing_config(edge_id)
        global_config = routing_config['routing']['routingGlobalConfig']
        global_config['ecmp'] = False
        global_config.pop('routerId')
        global_config.pop('ipPrefixes')
        self.vcns.update_edge_routing_config(edge_id, routing_config)

    def get_routing_bgp_config(self, edge_id):
        h, config = self.vcns.get_bgp_routing_config(edge_id)
        bgp_config = config if config else {}
        self._prepare_bgp_config(bgp_config)
        return {'bgp': bgp_config}

    def _update_bgp_routing_config(self, edge_id, **kwargs):
        bgp_config = self.get_routing_bgp_config(edge_id)
        curr_neighbours = bgp_config['bgp']['bgpNeighbours']
        curr_rules = bgp_config['bgp']['redistribution']['rules']

        bgp_config['bgp']['enabled'] = True

        if 'default_originate' in kwargs:
            bgp_config['bgp']['defaultOriginate'] = kwargs['default_originate']

        if 'local_as' in kwargs:
            bgp_config['bgp']['localAS'] = kwargs['local_as']

        if 'enabled' in kwargs:
            bgp_config['bgp']['redistribution']['enabled'] = kwargs['enabled']

        curr_rules[:] = [rule for rule in curr_rules
                         if rule['rule'].get('prefixName') not in
                         kwargs.get('rules_to_remove', [])]
        # Avoid adding duplicate rules when shared router relocation
        curr_rules_prefixes = [r['rule'].get('prefixName') for r in curr_rules]
        curr_rules.extend([r for r in kwargs.get('rules_to_add', [])
                           if r['rule'].get('prefixName') not in
                           curr_rules_prefixes])

        neighbours_to_remove = [nbr['bgpNeighbour']['ipAddress'] for nbr in
                                kwargs.get('neighbours_to_remove', [])]
        curr_neighbours[:] = [nbr for nbr in curr_neighbours
                              if nbr['bgpNeighbour']['ipAddress']
                              not in neighbours_to_remove]
        curr_neighbours.extend(kwargs.get('neighbours_to_add', []))

        self.vcns.update_bgp_dynamic_routing(edge_id, bgp_config)

    def add_bgp_speaker_config(self, edge_id, prot_router_id, local_as,
                               enabled, bgp_neighbours,
                               prefixes, redistribution_rules,
                               default_originate=False):
        with locking.LockManager.get_lock(str(edge_id)):
            self._update_routing_config(edge_id,
                                        router_id=prot_router_id,
                                        prefixes_to_add=prefixes)
            if self.ecmp_wait_time > 0:
                time.sleep(self.ecmp_wait_time)
            self._update_bgp_routing_config(
                edge_id, enabled=enabled, local_as=local_as,
                neighbours_to_add=bgp_neighbours, prefixes_to_add=prefixes,
                rules_to_add=redistribution_rules,
                default_originate=default_originate)

    def delete_bgp_speaker_config(self, edge_id):
        with locking.LockManager.get_lock(str(edge_id)):
            self.vcns.delete_bgp_routing_config(edge_id)
            self._reset_routing_global_config(edge_id)

    def add_bgp_neighbours(self, edge_id, bgp_neighbours):
        # Query the bgp config first and update the bgpNeighbour
        with locking.LockManager.get_lock(str(edge_id)):
            self._update_bgp_routing_config(edge_id,
                                            neighbours_to_add=bgp_neighbours)

    def remove_bgp_neighbours(self, edge_id, bgp_neighbours):
        with locking.LockManager.get_lock(str(edge_id)):
            self._update_bgp_routing_config(
                edge_id, neighbours_to_remove=bgp_neighbours)

    def update_bgp_neighbours(self, edge_id, neighbours_to_add=None,
                              neighbours_to_remove=None):
        with locking.LockManager.get_lock(str(edge_id)):
            self._update_bgp_routing_config(
                edge_id,
                neighbours_to_add=neighbours_to_add,
                neighbours_to_remove=neighbours_to_remove)

    def update_routing_redistribution(self, edge_id, enabled):
        with locking.LockManager.get_lock(str(edge_id)):
            self._update_bgp_routing_config(edge_id, enabled=enabled)

    def add_bgp_redistribution_rules(self, edge_id, prefixes, rules):
        with locking.LockManager.get_lock(str(edge_id)):
            self._update_routing_config(edge_id, prefixes_to_add=prefixes)
            self._update_bgp_routing_config(edge_id, rules_to_add=rules)
        LOG.debug("Added redistribution rules %s on edge %s", rules, edge_id)

    def remove_bgp_redistribution_rules(self, edge_id, prefixes):
        with locking.LockManager.get_lock(str(edge_id)):
            self._update_bgp_routing_config(edge_id, rules_to_remove=prefixes)
            self._update_routing_config(edge_id, prefixes_to_remove=prefixes)
        LOG.debug("Removed redistribution rules for prefixes %s on edge %s",
                  prefixes, edge_id)

    def update_router_id(self, edge_id, router_id):
        with locking.LockManager.get_lock(str(edge_id)):
            self._update_routing_config(edge_id, router_id=router_id)



# Copyright 2013 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from oslo_log import log as logging

from neutron.db import _resource_extend as resource_extend

from vmware_nsx.db import nsxv_models

LOG = logging.getLogger(__name__)


@resource_extend.has_resource_extenders
class NsxRouterMixin(object):
    """Mixin class to enable nsx router support."""

    nsx_attributes = []

    @staticmethod
    def _extend_nsx_router_dict(router_res, router_db, nsx_attributes):
        nsx_attrs = router_db['nsx_attributes']
        for attr in nsx_attributes:
            name = attr['name']
            default = attr['default']
            router_res[name] = (
                nsx_attrs and nsx_attrs[name] or default)

    def _process_nsx_router_create(
        self, context, router_db, router_req):
        if not router_db['nsx_attributes']:
            kwargs = {}
            for attr in self.nsx_attributes:
                name = attr['name']
                default = attr['default']
                kwargs[name] = router_req.get(name, default)
            nsx_attributes = nsxv_models.NsxvRouterExtAttributes(
                router_id=router_db['id'], **kwargs)
            context.session.add(nsx_attributes)
            router_db['nsx_attributes'] = nsx_attributes
        else:
            # The situation where the record already exists will
            # be likely once the NSXRouterExtAttributes model
            # will allow for defining several attributes pertaining
            # to different extensions
            for attr in self.nsx_attributes:
                name = attr['name']
                default = attr['default']
                router_db['nsx_attributes'][name] = router_req.get(
                    name, default)
        LOG.debug("Nsx router extension successfully processed "
                  "for router:%s", router_db['id'])



# Copyright 2016 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from vmware_nsx.shell.admin.plugins.common import constants
from vmware_nsx.shell.admin.plugins.common import formatters
import vmware_nsx.shell.admin.plugins.common.utils as admin_utils
import vmware_nsx.shell.admin.plugins.nsxv.resources.utils as utils
import vmware_nsx.shell.resources as shell

from neutron_lib.callbacks import registry
from neutron_lib import context as n_context
from oslo_config import cfg
from oslo_log import log as logging

from vmware_nsx.common import locking
from vmware_nsx.db import nsxv_db
from vmware_nsx.extensions import routersize
from vmware_nsx.plugins.nsx_v import availability_zones as nsx_az
from vmware_nsx.plugins.nsx_v.vshield import edge_utils
from vmware_nsx.plugins.nsx_v.vshield import vcns_driver

LOG = logging.getLogger(__name__)


def delete_old_edge(context, old_edge_id):
    LOG.info("Deleting the old edge: %s", old_edge_id)

    # clean it up from the DB
    nsxv_db.clean_edge_router_binding(context.session, old_edge_id)
    nsxv_db.clean_edge_vnic_binding(context.session, old_edge_id)
    nsxv_db.cleanup_nsxv_edge_firewallrule_binding(context.session,
                                                   old_edge_id)

    with locking.LockManager.get_lock(old_edge_id):
        # Delete from NSXv backend
        # Note - If we will not delete the edge, but free it - it will be
        # immediately used as the new one, So it is better to delete it.
        try:
            nsxv = utils.get_nsxv_client()
            nsxv.delete_edge(old_edge_id)
        except Exception as e:
            LOG.warning("Failed to delete the old edge %(id)s: %(e)s",
                        {'id': old_edge_id, 'e': e})
            # Continue the process anyway
            # The edge may have been already deleted at the backend


def _get_router_az_from_plugin_router(router):
    # If the router edge was already deployed the availability_zones will
    # return the az
    az_name = router.get('availability_zones', [''])[0]
    if not az_name:
        # If it was not deployed - it may be in the creation hints
        az_name = router.get('availability_zones_hints', [''])[0]
    if not az_name:
        # If not - the default az was used.
        az_name = nsx_az.DEFAULT_NAME
    return az_name


def nsx_recreate_router_edge(old_edge_id):
    # init the plugin and edge manager
    cfg.CONF.set_override('core_plugin',
                          'vmware_nsx.shell.admin.plugins.nsxv.resources'
                          '.utils.NsxVPluginWrapper')
    with utils.NsxVPluginWrapper() as plugin:
        nsxv_manager = vcns_driver.VcnsDriver(
            edge_utils.NsxVCallbacks(plugin))
        edge_manager = edge_utils.EdgeManager(nsxv_manager, plugin)
        context = n_context.get_admin_context()

        # verify that this is a Router edge
        router_ids = edge_manager.get_routers_on_edge(context, old_edge_id)
        if not router_ids:
            LOG.error("Edge %(edge_id)s is not a router edge",
                     {'edge_id': old_edge_id})
            return

        # all the routers on the same edge have the same type, so it
        # is ok to check the type once
        example_router = plugin.get_router(context, router_ids[0])
        if example_router.get('distributed'):
            LOG.error("Recreating a distributed router edge is not "
                      "supported")
            return
        router_driver = plugin._router_managers.get_tenant_router_driver(
            context, example_router['router_type'])

        # load all the routers before deleting their binding
        routers = []
        for router_id in router_ids:
            routers.append(plugin.get_router(context, router_id))

        # delete the backend edge and all the relevant DB entries
        delete_old_edge(context, old_edge_id)

        # Go over all the relevant routers
        for router in routers:
            router_id = router['id']
            az_name = _get_router_az_from_plugin_router(router)
            # clean up other objects related to this router
            if plugin.metadata_proxy_handler:
                md_proxy = plugin.get_metadata_proxy_handler(az_name)
                md_proxy.cleanup_router_edge(context, router_id)

            # attach the router to a new edge
            appliance_size = router.get(routersize.ROUTER_SIZE)
            router_driver.attach_router(context, router_id,
                                        {'router': router},
                                        appliance_size=appliance_size)
            # find out who is the new edge to print it
            new_edge_id = router_driver._get_edge_id_or_raise(
                context, router_id)
            LOG.info("Router %(router)s was attached to edge %(edge)s",
                     {'router': router_id, 'edge': new_edge_id})


def nsx_recreate_router(router_id):
    # init the plugin and edge manager
    cfg.CONF.set_override('core_plugin',
                          'vmware_nsx.shell.admin.plugins.nsxv.resources'
                          '.utils.NsxVPluginWrapper')
    with utils.NsxVPluginWrapper() as plugin:
        context = n_context.get_admin_context()

        router = plugin.get_router(context, router_id)
        if router.get('distributed'):
            LOG.error("Recreating a distributed router is not supported")
            return
        router_driver = plugin._router_managers.get_tenant_router_driver(
            context, router['router_type'])

        # Check if it is already attached to an edge
        binding = nsxv_db.get_nsxv_router_binding(context.session,
                                                  router_id)
        if binding:
            old_edge_id = binding['edge_id']
            # detach the router from this edge
            LOG.info("Detaching the router from edge %s", old_edge_id)
            router_driver.detach_router(context, router_id,
                                        {'router': router})

        # attach the router to a new edge
        appliance_size = router.get(routersize.ROUTER_SIZE)
        router_driver.attach_router(context, router_id,
                                    {'router': router},
                                    appliance_size=appliance_size)
        # find out who is the new edge to print it
        new_edge_id = router_driver._get_edge_id_or_raise(
            context, router_id)
        LOG.info("Router %(router)s was attached to edge %(edge)s",
                 {'router': router_id, 'edge': new_edge_id})


@admin_utils.output_header
def nsx_recreate_router_or_edge(resource, event, trigger, **kwargs):
    """Recreate a router edge with all the data on a new NSXv edge"""
    if not kwargs.get('property'):
        LOG.error("Need to specify edge-id or router-id parameter")
        return

    # input validation
    properties = admin_utils.parse_multi_keyval_opt(kwargs['property'])
    old_edge_id = properties.get('edge-id')
    router_id = properties.get('router-id')
    if (not old_edge_id and not router_id) or (old_edge_id and router_id):
        LOG.error("Need to specify edge-id or router-id parameter")
        return

    if old_edge_id:
        LOG.info("ReCreating NSXv Router Edge: %s", old_edge_id)
        return nsx_recreate_router_edge(old_edge_id)
    else:
        LOG.info("ReCreating NSXv Router: %s", router_id)
        return nsx_recreate_router(router_id)


@admin_utils.output_header
def migrate_distributed_routers_dhcp(resource, event, trigger, **kwargs):
    context = n_context.get_admin_context()
    nsxv = utils.get_nsxv_client()
    with utils.NsxVPluginWrapper() as plugin:
        routers = plugin.get_routers(context)
        for router in routers:
            if router.get('distributed', False):
                binding = nsxv_db.get_nsxv_router_binding(context.session,
                                                          router['id'])
                if binding:
                    edge_id = binding['edge_id']
                    with locking.LockManager.get_lock(edge_id):
                        route_obj = nsxv.get_routes(edge_id)[1]
                        routes = route_obj.get('staticRoutes', {}
                                               ).get('staticRoutes', [])
                        new_routes = [route for route in routes if route.get(
                            'network') != '169.254.169.254/32']
                        route_obj['staticRoutes']['staticRoutes'] = new_routes

                        nsxv.update_routes(edge_id, route_obj)


def is_router_conflicting_on_edge(context, driver, router_id):
    edge_id = edge_utils.get_router_edge_id(context, router_id)
    if not edge_id:
        return False
    (available_routers,
     conflict_routers) = driver._get_available_and_conflicting_ids(
        context, router_id)
    for conf_router in conflict_routers:
        conf_edge_id = edge_utils.get_router_edge_id(context, conf_router)
        if conf_edge_id == edge_id:
            LOG.info("Router %(rtr)s on edge %(edge)s is conflicting with "
                     "another router and will be moved",
                     {'rtr': router_id, 'edge': edge_id})
            return True
    return False


@admin_utils.output_header
def redistribute_routers(resource, event, trigger, **kwargs):
    """If any of the shared routers are on a conflicting edge move them"""
    context = n_context.get_admin_context()
    with utils.NsxVPluginWrapper() as plugin:
        router_driver = plugin._router_managers.get_tenant_router_driver(
            context, 'shared')
        routers = plugin.get_routers(context)
        for router in routers:
            if (not router.get('distributed', False) and
                router.get('router_type') == 'shared' and
                is_router_conflicting_on_edge(
                    context, router_driver, router['id'])):
                router_driver.detach_router(context, router['id'], router)
                router_driver.attach_router(context, router['id'], router)


@admin_utils.output_header
def list_orphaned_vnics(resource, event, trigger, **kwargs):
    """List router orphaned router vnics where the port was deleted"""
    orphaned_vnics = get_orphaned_vnics()
    if not orphaned_vnics:
        LOG.info("No orphaned router vnics found")
        return
    headers = ['edge_id', 'vnic_index', 'tunnel_index', 'network_id']
    LOG.info(formatters.output_formatter(constants.ORPHANED_VNICS,
                                         orphaned_vnics, headers))


def get_orphaned_vnics():
    orphaned_vnics = []
    context = n_context.get_admin_context()
    vnic_binds = nsxv_db.get_edge_vnic_bindings_with_networks(
        context.session)
    with utils.NsxVPluginWrapper() as plugin:
        for vnic_bind in vnic_binds:
            edge_id = vnic_bind['edge_id']
            # check if this is a router edge by the router bindings table
            router_bindings = nsxv_db.get_nsxv_router_bindings_by_edge(
                context.session, edge_id)
            if not router_bindings:
                # Only log it. this is a different type of orphaned
                LOG.warning("Router bindings for vnic %s not found", vnic_bind)
                continue

            router_ids = [b['router_id'] for b in router_bindings]
            routers = plugin.get_routers(context,
                                         filters={'id': router_ids})
            if routers:
                interface_found = False
                # check if any of those routers is attached to this network
                for router in routers:
                    if plugin._get_router_interface_ports_by_network(
                        context, router['id'], vnic_bind['network_id']):
                        interface_found = True
                        break
                if not interface_found:
                    # for later deleting the interface we need to know if this
                    # is a distributed router.
                    # All the routers on the same edge are of the same type,
                    # so we can check the first one.
                    vnic_bind['distributed'] = routers[0].get('distributed')
                    orphaned_vnics.append(vnic_bind)

    return orphaned_vnics


@admin_utils.output_header
def clean_orphaned_vnics(resource, event, trigger, **kwargs):
    """List router orphaned router vnics where the port was deleted"""
    orphaned_vnics = get_orphaned_vnics()
    if not orphaned_vnics:
        LOG.info("No orphaned router vnics found")
        return
    headers = ['edge_id', 'vnic_index', 'tunnel_index', 'network_id']
    LOG.info(formatters.output_formatter(constants.ORPHANED_VNICS,
                                         orphaned_vnics, headers))
    user_confirm = admin_utils.query_yes_no("Do you want to delete "
                                            "orphaned vnics",
                                            default="no")
    if not user_confirm:
        LOG.info("NSXv vnics deletion aborted by user")
        return

    context = n_context.get_admin_context()
    with utils.NsxVPluginWrapper() as plugin:
        nsxv_manager = vcns_driver.VcnsDriver(
            edge_utils.NsxVCallbacks(plugin))
        for vnic in orphaned_vnics:
            if not vnic['distributed']:
                try:
                    nsxv_manager.vcns.delete_interface(
                        vnic['edge_id'], vnic['vnic_index'])
                except Exception as e:
                    LOG.error("Failed to delete vnic from NSX: %s", e)
                nsxv_db.free_edge_vnic_by_network(
                    context.session, vnic['edge_id'], vnic['network_id'])
            else:
                try:
                    nsxv_manager.vcns.delete_vdr_internal_interface(
                        vnic['edge_id'], vnic['vnic_index'])
                except Exception as e:
                    LOG.error("Failed to delete vnic from NSX: %s", e)
                nsxv_db.delete_edge_vnic_binding_by_network(
                    context.session, vnic['edge_id'], vnic['network_id'])


registry.subscribe(nsx_recreate_router_or_edge,
                   constants.ROUTERS,
                   shell.Operations.NSX_RECREATE.value)

registry.subscribe(migrate_distributed_routers_dhcp,
                   constants.ROUTERS,
                   shell.Operations.MIGRATE_VDR_DHCP.value)

registry.subscribe(redistribute_routers,
                   constants.ROUTERS,
                   shell.Operations.NSX_REDISTRIBURE.value)

registry.subscribe(list_orphaned_vnics,
                   constants.ORPHANED_VNICS,
                   shell.Operations.NSX_LIST.value)

registry.subscribe(clean_orphaned_vnics,
                   constants.ORPHANED_VNICS,
                   shell.Operations.NSX_CLEAN.value)



# Copyright 2017 VMware, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""nsxv3_lbaas_l7policy

Revision ID: 717f7f63a219
Revises: a1be06050b41
Create Date: 2017-10-26 08:32:40.846088

"""

# revision identifiers, used by Alembic.
revision = '717f7f63a219'
down_revision = 'a1be06050b41'

from alembic import op
import sqlalchemy as sa

from neutron.db import migration


def upgrade():

    if migration.schema_has_table('nsxv3_lbaas_l7rules'):
        op.drop_constraint('fk_nsxv3_lbaas_l7rules_id', 'nsxv3_lbaas_l7rules',
                           'foreignkey')
        op.drop_constraint('l7rule_id', 'nsxv3_lbaas_l7rules', 'primary')
        op.drop_column('nsxv3_lbaas_l7rules', 'loadbalancer_id')
        op.drop_column('nsxv3_lbaas_l7rules', 'l7rule_id')
        op.rename_table('nsxv3_lbaas_l7rules', 'nsxv3_lbaas_l7policies')

        if migration.schema_has_table('lbaas_l7policies'):
            op.create_foreign_key(
                'fk_nsxv3_lbaas_l7policies_id', 'nsxv3_lbaas_l7policies',
                'lbaas_l7policies', ['l7policy_id'], ['id'],
                ondelete='CASCADE')
    else:
        op.create_table(
            'nsxv3_lbaas_l7policies',
            sa.Column('l7policy_id', sa.String(36), nullable=False),
            sa.Column('lb_rule_id', sa.String(36), nullable=False),
            sa.Column('lb_vs_id', sa.String(36), nullable=False),
            sa.Column('created_at', sa.DateTime(), nullable=True),
            sa.Column('updated_at', sa.DateTime(), nullable=True),
            sa.PrimaryKeyConstraint('l7policy_id'))

        if migration.schema_has_table('lbaas_l7policies'):
            op.create_foreign_key(
                'fk_nsxv3_lbaas_l7policies_id', 'nsxv3_lbaas_l7policies',
                'lbaas_l7policies', ['l7policy_id'], ['id'],
                ondelete='CASCADE')



# Copyright 2016 VMware, Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import contextlib

import mock

from neutron_lib.api.definitions import external_net as extnet_apidef
from neutron_lib import context
from neutron_lib.plugins import directory
from neutron_vpnaas.db.vpn import vpn_models  # noqa
from neutron_vpnaas.extensions import vpnaas
from oslo_utils import uuidutils

from vmware_nsx.common import exceptions as nsxv_exc
from vmware_nsx.plugins.nsx_v.vshield.common import exceptions as vcns_exc
from vmware_nsx.services.vpnaas.nsxv import ipsec_driver
from vmware_nsx.tests.unit.nsx_v import test_plugin


_uuid = uuidutils.generate_uuid

DRIVER_PATH = "vmware_nsx.services.vpnaas.nsxv.ipsec_driver.NSXvIPsecVpnDriver"
VALI_PATH = "vmware_nsx.services.vpnaas.nsxv.ipsec_validator.IPsecValidator"
FAKE_ROUTER_ID = "aaaaaa-bbbbb-ccc"
FAKE_VPNSERVICE_ID = _uuid()
FAKE_IPSEC_CONNECTION = {"vpnservice_id": FAKE_VPNSERVICE_ID,
                         "id": _uuid()}
FAKE_EDGE_ID = _uuid()
FAKE_IPSEC_VPN_SITE = {"peerIp": "192.168.1.1"}
FAKE_VCNSAPIEXC = {"status": "fail",
                   "head": "fake_head",
                   "response": "error"}
FAKE_NEW_CONNECTION = {"peer_cidrs": "192.168.1.0/24"}


class TestVpnaasDriver(test_plugin.NsxVPluginV2TestCase):

    def setUp(self):
        super(TestVpnaasDriver, self).setUp()
        self.context = context.get_admin_context()
        self.service_plugin = mock.Mock()
        self.validator = mock.Mock()
        self.driver = ipsec_driver.NSXvIPsecVpnDriver(self.service_plugin)
        self.plugin = directory.get_plugin()
        self.l3plugin = self.plugin

    @contextlib.contextmanager
    def router(self, name='vpn-test-router', tenant_id=_uuid(),
               admin_state_up=True, **kwargs):
        request = {'router': {'tenant_id': tenant_id,
                              'name': name,
                              'admin_state_up': admin_state_up}}
        for arg in kwargs:
            request['router'][arg] = kwargs[arg]
        router = self.l3plugin.create_router(self.context, request)
        yield router

    @mock.patch('%s._convert_ipsec_conn' % DRIVER_PATH)
    @mock.patch('%s._get_router_edge_id' % DRIVER_PATH)
    @mock.patch('%s._generate_new_sites' % DRIVER_PATH)
    @mock.patch('%s._update_ipsec_config' % DRIVER_PATH)
    @mock.patch('%s._update_status' % DRIVER_PATH)
    @mock.patch('%s._update_firewall_rules' % DRIVER_PATH)
    def test_create_ipsec_site_connection(self, mock_update_fw,
                                          mock_update_status,
                                          mock_update_ipsec, mock_gen_new,
                                          mock_get_id,
                                          mock_conv_ipsec):
        mock_get_id.return_value = (FAKE_ROUTER_ID, FAKE_EDGE_ID)
        mock_conv_ipsec.return_value = FAKE_IPSEC_VPN_SITE
        mock_gen_new.return_value = FAKE_IPSEC_VPN_SITE
        self.driver.create_ipsec_site_connection(self.context,
                                                 FAKE_IPSEC_CONNECTION)
        mock_conv_ipsec.assert_called_with(self.context,
                                           FAKE_IPSEC_CONNECTION)
        mock_get_id.assert_called_with(self.context, FAKE_VPNSERVICE_ID)
        mock_gen_new.assert_called_with(FAKE_EDGE_ID, FAKE_IPSEC_VPN_SITE)
        mock_update_ipsec.assert_called_with(FAKE_EDGE_ID,
                                             FAKE_IPSEC_VPN_SITE,
                                             enabled=True)
        mock_update_fw.assert_called_with(self.context, FAKE_VPNSERVICE_ID)
        mock_update_status.assert_called_with(
            self.context,
            FAKE_IPSEC_CONNECTION["vpnservice_id"],
            FAKE_IPSEC_CONNECTION["id"],
            "ACTIVE")

    @mock.patch('%s._convert_ipsec_conn' % DRIVER_PATH)
    @mock.patch('%s._get_router_edge_id' % DRIVER_PATH)
    @mock.patch('%s._generate_new_sites' % DRIVER_PATH)
    @mock.patch('%s._update_ipsec_config' % DRIVER_PATH)
    @mock.patch('%s._update_status' % DRIVER_PATH)
    def test_create_ipsec_site_connection_fail(self,
                                               mock_update_status,
                                               mock_update_ipsec,
                                               mock_gen_new, mock_get_id,
                                               mock_conv_ipsec):
        mock_get_id.return_value = (FAKE_ROUTER_ID, FAKE_EDGE_ID)
        mock_conv_ipsec.return_value = FAKE_IPSEC_VPN_SITE
        mock_gen_new.return_value = FAKE_IPSEC_VPN_SITE
        mock_update_ipsec.side_effect = (
            vcns_exc.VcnsApiException(**FAKE_VCNSAPIEXC))
        self.assertRaises(nsxv_exc.NsxPluginException,
                          self.driver.create_ipsec_site_connection,
                          self.context, FAKE_IPSEC_CONNECTION)
        mock_conv_ipsec.assert_called_with(self.context, FAKE_IPSEC_CONNECTION)
        mock_get_id.assert_called_with(self.context, FAKE_VPNSERVICE_ID)
        mock_gen_new.assert_called_with(FAKE_EDGE_ID, FAKE_IPSEC_VPN_SITE)
        mock_update_ipsec.assert_called_with(FAKE_EDGE_ID,
                                             FAKE_IPSEC_VPN_SITE,
                                             enabled=True)
        mock_update_status.assert_called_with(
            self.context,
            FAKE_IPSEC_CONNECTION["vpnservice_id"],
            FAKE_IPSEC_CONNECTION["id"],
            "ERROR")

    @mock.patch('%s._convert_ipsec_conn' % DRIVER_PATH)
    @mock.patch('%s._get_router_edge_id' % DRIVER_PATH)
    @mock.patch('%s._generate_new_sites' % DRIVER_PATH)
    @mock.patch('%s._update_ipsec_config' % DRIVER_PATH)
    @mock.patch('%s._update_status' % DRIVER_PATH)
    @mock.patch('%s._update_firewall_rules' % DRIVER_PATH)
    def test_update_fw_fail(self, mock_update_fw, mock_update_status,
                            mock_update_ipsec, mock_gen_new,
                            mock_get_id, mock_conv_ipsec):
        mock_get_id.return_value = (FAKE_ROUTER_ID, FAKE_EDGE_ID)
        mock_conv_ipsec.return_value = FAKE_IPSEC_VPN_SITE
        mock_gen_new.return_value = FAKE_IPSEC_VPN_SITE
        mock_update_fw.side_effect = (
            vcns_exc.VcnsApiException(**FAKE_VCNSAPIEXC))
        self.assertRaises(nsxv_exc.NsxPluginException,
                          self.driver.create_ipsec_site_connection,
                          self.context, FAKE_IPSEC_CONNECTION)
        mock_conv_ipsec.assert_called_with(self.context, FAKE_IPSEC_CONNECTION)
        mock_get_id.assert_called_with(self.context, FAKE_VPNSERVICE_ID)
        mock_gen_new.assert_called_with(FAKE_EDGE_ID, FAKE_IPSEC_VPN_SITE)
        mock_update_ipsec.assert_called_with(FAKE_EDGE_ID,
                                             FAKE_IPSEC_VPN_SITE,
                                             enabled=True)
        mock_update_fw.assert_called_with(self.context, FAKE_VPNSERVICE_ID)
        mock_update_status.assert_called_with(
            self.context,
            FAKE_IPSEC_CONNECTION["vpnservice_id"],
            FAKE_IPSEC_CONNECTION["id"],
            "ERROR")

    @mock.patch('%s._get_router_edge_id' % DRIVER_PATH)
    @mock.patch('%s._update_site_dict' % DRIVER_PATH)
    @mock.patch('%s._update_ipsec_config' % DRIVER_PATH)
    @mock.patch('%s._update_firewall_rules' % DRIVER_PATH)
    def test_update_ipsec(self, mock_update_fw, mock_update_ipsec,
                          mock_update_sites, mock_get_id):
        mock_get_id.return_value = (FAKE_ROUTER_ID, FAKE_EDGE_ID)
        mock_update_sites.return_value = FAKE_IPSEC_VPN_SITE
        self.driver.update_ipsec_site_connection(self.context,
                                                 FAKE_IPSEC_CONNECTION,
                                                 FAKE_NEW_CONNECTION)
        mock_update_sites.assert_called_with(self.context, FAKE_EDGE_ID,
                                             FAKE_IPSEC_CONNECTION,
                                             FAKE_NEW_CONNECTION)
        mock_update_ipsec.assert_called_with(FAKE_EDGE_ID, FAKE_IPSEC_VPN_SITE)
        mock_update_fw.assert_called_with(self.context, FAKE_VPNSERVICE_ID)

    @mock.patch('%s._get_router_edge_id' % DRIVER_PATH)
    @mock.patch('%s._update_site_dict' % DRIVER_PATH)
    @mock.patch('%s._update_ipsec_config' % DRIVER_PATH)
    @mock.patch('%s._update_firewall_rules' % DRIVER_PATH)
    def test_update_ipsec_fail_with_notfound(self, mock_update_fw,
                                             mock_update_ipsec,
                                             mock_update_sites, mock_get_id):
        mock_get_id.return_value = (FAKE_ROUTER_ID, FAKE_EDGE_ID)
        mock_update_sites.return_value = {}
        self.assertRaises(nsxv_exc.NsxIPsecVpnMappingNotFound,
                          self.driver.update_ipsec_site_connection,
                          self.context, FAKE_IPSEC_CONNECTION,
                          FAKE_NEW_CONNECTION)
        mock_update_sites.assert_called_with(self.context,
                                             FAKE_EDGE_ID,
                                             FAKE_IPSEC_CONNECTION,
                                             FAKE_NEW_CONNECTION)

    @mock.patch('%s._get_router_edge_id' % DRIVER_PATH)
    @mock.patch('%s._update_site_dict' % DRIVER_PATH)
    @mock.patch('%s._update_ipsec_config' % DRIVER_PATH)
    @mock.patch('%s._update_firewall_rules' % DRIVER_PATH)
    def test_update_ipsec_fail_with_fw_fail(self, mock_update_fw,
                                            mock_update_ipsec,
                                            mock_update_sites, mock_get_id):
        mock_get_id.return_value = (FAKE_ROUTER_ID, FAKE_EDGE_ID)
        mock_update_fw.side_effect = (
            vcns_exc.VcnsApiException(**FAKE_VCNSAPIEXC))
        self.assertRaises(nsxv_exc.NsxPluginException,
                          self.driver.update_ipsec_site_connection,
                          self.context, FAKE_IPSEC_CONNECTION,
                          FAKE_NEW_CONNECTION)
        mock_update_sites.assert_called_with(self.context, FAKE_EDGE_ID,
                                             FAKE_IPSEC_CONNECTION,
                                             FAKE_NEW_CONNECTION)
        mock_update_fw.assert_called_with(self.context, FAKE_VPNSERVICE_ID)

    @mock.patch('%s._get_router_edge_id' % DRIVER_PATH)
    @mock.patch('%s._update_site_dict' % DRIVER_PATH)
    @mock.patch('%s._update_ipsec_config' % DRIVER_PATH)
    @mock.patch('%s._update_status' % DRIVER_PATH)
    def test_update_ipsec_fail_with_site_fail(self, mock_update_status,
                                              mock_update_ipsec,
                                              mock_update_sites, mock_get_id):
        mock_get_id.return_value = (FAKE_ROUTER_ID, FAKE_EDGE_ID)
        mock_update_sites.return_value = FAKE_IPSEC_VPN_SITE
        mock_update_ipsec.side_effect = (
            vcns_exc.VcnsApiException(**FAKE_VCNSAPIEXC))
        self.assertRaises(nsxv_exc.NsxPluginException,
                          self.driver.update_ipsec_site_connection,
                          self.context,
                          FAKE_IPSEC_CONNECTION,
                          FAKE_NEW_CONNECTION)
        mock_update_sites.assert_called_with(self.context, FAKE_EDGE_ID,
                                             FAKE_IPSEC_CONNECTION,
                                             FAKE_NEW_CONNECTION)
        mock_update_ipsec.assert_called_with(FAKE_EDGE_ID,
                                             FAKE_IPSEC_VPN_SITE)
        mock_update_status.assert_called_with(
            self.context,
            FAKE_IPSEC_CONNECTION["vpnservice_id"],
            FAKE_IPSEC_CONNECTION["id"],
            "ERROR")

    def test_create_vpn_service_legal(self):
        """Create a legal vpn service"""
        # create an external network with a subnet, and an exclusive router
        providernet_args = {extnet_apidef.EXTERNAL: True}
        with self.network(name='ext-net',
                          providernet_args=providernet_args,
                          arg_list=(extnet_apidef.EXTERNAL, )) as ext_net,\
            self.subnet(ext_net),\
            self.router(router_type='exclusive',
                        external_gateway_info={'network_id':
                            ext_net['network']['id']}) as router,\
            self.subnet() as sub:
            # add an interface to the router
            self.l3plugin.add_router_interface(
                self.context,
                router['id'],
                {'subnet_id': sub['subnet']['id']})
            # create the service
            vpnservice = {'router_id': router['id'],
                          'id': _uuid(),
                          'subnet_id': sub['subnet']['id']}
            with mock.patch.object(self.driver, '_get_gateway_ips',
                                   return_value=(None, None)):
                self.driver.create_vpnservice(self.context, vpnservice)

    def test_create_vpn_service_on_shared_router(self):
        """Creating a service with shared router is not allowed"""
        # create an external network with a subnet, and a shared router
        providernet_args = {extnet_apidef.EXTERNAL: True}
        with self.network(name='ext-net',
                          providernet_args=providernet_args,
                          arg_list=(extnet_apidef.EXTERNAL, )) as ext_net,\
            self.subnet(ext_net),\
            self.router(router_type='shared',
                        external_gateway_info={'network_id':
                            ext_net['network']['id']}) as router,\
            self.subnet() as sub:
            # add an interface to the router
            self.l3plugin.add_router_interface(
                self.context,
                router['id'],
                {'subnet_id': sub['subnet']['id']})
            # create the service
            vpnservice = {'router_id': router['id'],
                          'id': _uuid(),
                          'subnet_id': sub['subnet']['id']}
            self.assertRaises(nsxv_exc.NsxPluginException,
                              self.driver.create_vpnservice,
                              self.context, vpnservice)

    def test_create_vpn_service_on_router_without_if(self):
        """Creating a service with unattached subnet is not allowed"""
        # create an external network with a subnet, and an exclusive router
        providernet_args = {extnet_apidef.EXTERNAL: True}
        with self.network(name='ext-net',
                          providernet_args=providernet_args,
                          arg_list=(extnet_apidef.EXTERNAL, )) as ext_net,\
            self.subnet(ext_net),\
            self.router(router_type='exclusive',
                        external_gateway_info={'network_id':
                            ext_net['network']['id']}) as router,\
            self.subnet() as sub:
            # create the service
            vpnservice = {'router_id': router['id'],
                          'id': _uuid(),
                          'subnet_id': sub['subnet']['id']}
            self.assertRaises(vpnaas.SubnetIsNotConnectedToRouter,
                              self.driver.create_vpnservice,
                              self.context, vpnservice)

    def test_create_vpn_service_without_subnet(self):
        """Creating a service without a subnet is not allowed"""
        # create an external network with a subnet, and an exclusive router
        providernet_args = {extnet_apidef.EXTERNAL: True}
        with self.network(name='ext-net',
                          providernet_args=providernet_args,
                          arg_list=(extnet_apidef.EXTERNAL, )) as ext_net,\
            self.subnet(ext_net),\
            self.router(router_type='exclusive',
                        external_gateway_info={'network_id':
                            ext_net['network']['id']}) as router,\
            self.subnet() as sub:
            # add an interface to the router
            self.l3plugin.add_router_interface(
                self.context,
                router['id'],
                {'subnet_id': sub['subnet']['id']})
            # create the service without the subnet
            vpnservice = {'router_id': router['id'],
                          'id': _uuid(),
                          'subnet_id': None}
            self.assertRaises(nsxv_exc.NsxPluginException,
                              self.driver.create_vpnservice,
                              self.context, vpnservice)



# Copyright 2016 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

import mock
from openstackclient.tests.unit.network.v2 import test_router
from openstackclient.tests.unit import utils as tests_utils

from vmware_nsx.extensions import routersize
from vmware_nsx.extensions import routertype
from vmware_nsx.osc.v2 import router


supported_extensions = ('nsxv-router-size', 'nsxv-router-type')


class TestCreateRouter(test_router.TestCreateRouter):

    def setUp(self):
        super(TestCreateRouter, self).setUp()
        # Get the command object to test
        self.cmd = router.NsxCreateRouter(self.app, self.namespace)
        # mock the relevant extensions
        get_ext = mock.patch('vmware_nsx.osc.v2.utils.get_extensions').start()
        get_ext.return_value = supported_extensions

    def _test_create_with_arg_and_val(self, arg_name, arg_val, is_valid=True):
        self.network.create_router.reset_mock()
        # add '--' to the arg name and change '_' to '-'
        conv_name = '--' + re.sub('_', '-', arg_name)
        arglist = [
            self.new_router.name,
            conv_name, arg_val
        ]
        verifylist = [
            ('name', self.new_router.name),
            (arg_name, arg_val),
            ('enable', True),
            ('distributed', False),
        ]
        if not is_valid:
            self.assertRaises(tests_utils.ParserException, self.check_parser,
                              self.cmd, arglist, verifylist)
            return

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = (self.cmd.take_action(parsed_args))
        self.network.create_router.assert_called_once_with(**{
            'admin_state_up': True,
            'name': self.new_router.name,
            arg_name: arg_val,
        })

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def _test_create_with_size(self, size, is_valid=True):
        self._test_create_with_arg_and_val('router_size', size, is_valid)

    def test_create_with_sizes(self):
        # check all router types
        for rtr_size in routersize.VALID_EDGE_SIZES:
            self._test_create_with_size(rtr_size)

    def test_create_with_illegal_size(self):
        self._test_create_with_size('illegal', is_valid=False)

    def _test_create_with_type(self, rtr_type, is_valid=True):
        self._test_create_with_arg_and_val('router_type', rtr_type, is_valid)

    def test_create_with_types(self):
        # check all router types
        for rtr_type in routertype.VALID_TYPES:
            self._test_create_with_type(rtr_type)

    def test_create_with_illegal_type(self):
        self._test_create_with_type('illegal', is_valid=False)


class TestSetRouter(test_router.TestSetRouter):

    def setUp(self):
        super(TestSetRouter, self).setUp()
        # Get the command object to test
        self.cmd = router.NsxSetRouter(self.app, self.namespace)
        # mock the relevant extensions
        get_ext = mock.patch('vmware_nsx.osc.v2.utils.get_extensions').start()
        get_ext.return_value = supported_extensions

    def _test_set_with_arg_and_val(self, arg_name, arg_val, is_valid=True):
        self.network.update_router.reset_mock()
        # add '--' to the arg name and change '_' to '-'
        conv_name = '--' + re.sub('_', '-', arg_name)
        arglist = [
            self._router.name,
            conv_name, arg_val
        ]
        verifylist = [
            ('router', self._router.name),
            (arg_name, arg_val)
        ]
        if not is_valid:
            self.assertRaises(tests_utils.ParserException, self.check_parser,
                              self.cmd, arglist, verifylist)
            return

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        attrs = {arg_name: arg_val}
        self.network.update_router.assert_called_once_with(
            self._router, **attrs)
        self.assertIsNone(result)

    def _test_set_size(self, size, is_valid=True):
        self._test_set_with_arg_and_val('router_size', size, is_valid)

    def test_set_sizes(self):
        # check all router types
        for rtr_size in routersize.VALID_EDGE_SIZES:
            self._test_set_size(rtr_size)

    def test_set_illegal_size(self):
        # check illegal size
        self._test_set_size('illegal', is_valid=False)

    def _test_set_type(self, rtr_type, is_valid=True):
        self._test_set_with_arg_and_val('router_type', rtr_type, is_valid)

    def test_set_types(self):
        # check all router types
        for rtr_type in routertype.VALID_TYPES:
            self._test_set_type(rtr_type)

    def test_set_illegal_type(self):
        self._test_set_type('illegal', is_valid=False)



# Copyright 2014 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
import stevedore

from oslo_log import log

from vmware_nsx._i18n import _
from vmware_nsx.common import exceptions as nsx_exc

LOG = log.getLogger(__name__)
ROUTER_TYPE_DRIVERS = ["distributed", "exclusive", "shared"]


class RouterTypeManager(stevedore.named.NamedExtensionManager):
    """Manage router segment types using drivers."""

    def __init__(self, plugin):
        # Mapping from type name to DriverManager
        self.drivers = {}

        LOG.info("Configured router type driver names: %s",
                 ROUTER_TYPE_DRIVERS)
        super(RouterTypeManager, self).__init__(
            'vmware_nsx.neutron.nsxv.router_type_drivers',
            ROUTER_TYPE_DRIVERS,
            invoke_on_load=True,
            invoke_args=(plugin,))
        LOG.info("Loaded type driver names: %s", self.names())
        self._register_types()
        self._check_tenant_router_types(cfg.CONF.nsxv.tenant_router_types)

    def _register_types(self):
        for ext in self:
            router_type = ext.obj.get_type()
            if router_type in self.drivers:
                LOG.error("Type driver '%(new_driver)s' ignored because "
                          "type driver '%(old_driver)s' is already "
                          "registered for type '%(type)s'",
                          {'new_driver': ext.name,
                           'old_driver': self.drivers[router_type].name,
                           'type': router_type})
            else:
                self.drivers[router_type] = ext
        LOG.info("Registered types: %s", self.drivers.keys())

    def _check_tenant_router_types(self, types):
        self.tenant_router_types = []
        for router_type in types:
            if router_type in self.drivers:
                self.tenant_router_types.append(router_type)
            else:
                msg = _("No type driver for tenant router_type: %s. "
                        "Service terminated!") % router_type
                LOG.error(msg)
                raise SystemExit(msg)
        LOG.info("Tenant router_types: %s", self.tenant_router_types)

    def get_tenant_router_driver(self, context, router_type):
        driver = self.drivers.get(router_type)
        if driver:
            return driver.obj
        raise nsx_exc.NoRouterAvailable()

    def decide_tenant_router_type(self, context, router_type=None):
        if router_type is None:
            for rt in self.tenant_router_types:
                driver = self.drivers.get(rt)
                if driver:
                    return rt
            raise nsx_exc.NoRouterAvailable()
        elif context.is_admin:
            driver = self.drivers.get(router_type)
            if driver:
                return router_type
        elif router_type in self.tenant_router_types:
            driver = self.drivers.get(router_type)
            if driver:
                return router_type
        raise nsx_exc.NoRouterAvailable()



# Copyright 2016 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.db import model_base
import sqlalchemy as sa
from sqlalchemy import orm

from neutron.db import _resource_extend as resource_extend
from neutron.db import api as db_api
from neutron.db.models import securitygroup
from neutron.extensions import securitygroup as ext_sg
from neutron_lib.api import validators
from neutron_lib import exceptions as nexception
from vmware_nsx._i18n import _
from vmware_nsx.extensions import secgroup_rule_local_ip_prefix as ext_local_ip


class NotIngressRule(nexception.BadRequest):
    message = _("Specifying local_ip_prefix is supported "
                "with ingress rules only.")


class NsxExtendedSecurityGroupRuleProperties(model_base.BASEV2):
    """Persist security group rule properties for the
    extended-security-group-rule extension.
    """

    __tablename__ = 'nsx_extended_security_group_rule_properties'

    rule_id = sa.Column(sa.String(36),
                        sa.ForeignKey('securitygrouprules.id',
                                      ondelete='CASCADE'),
                        primary_key=True,
                        nullable=False)
    local_ip_prefix = sa.Column(sa.String(255), nullable=False)

    rule = orm.relationship(
        securitygroup.SecurityGroupRule,
        backref=orm.backref('ext_properties', lazy='joined',
                            uselist=False, cascade='delete'))


@resource_extend.has_resource_extenders
class ExtendedSecurityGroupRuleMixin(object):

    def _check_local_ip_prefix(self, context, rule):
        rule_specify_local_ip_prefix = validators.is_attr_set(
            rule.get(ext_local_ip.LOCAL_IP_PREFIX))

        if rule_specify_local_ip_prefix and rule['direction'] != 'ingress':
            raise NotIngressRule()

        if not rule_specify_local_ip_prefix:
            # remove ATTR_NOT_SPECIFIED
            rule[ext_local_ip.LOCAL_IP_PREFIX] = None
        return rule_specify_local_ip_prefix

    def _process_security_group_rule_properties(self, context,
                                                rule_res, rule_req):
        rule_res[ext_local_ip.LOCAL_IP_PREFIX] = None
        if not validators.is_attr_set(
            rule_req.get(ext_local_ip.LOCAL_IP_PREFIX)):
            return

        with db_api.context_manager.writer.using(context):
            properties = NsxExtendedSecurityGroupRuleProperties(
                rule_id=rule_res['id'],
                local_ip_prefix=rule_req[ext_local_ip.LOCAL_IP_PREFIX])
            context.session.add(properties)
        rule_res[ext_local_ip.LOCAL_IP_PREFIX] = (
            rule_req[ext_local_ip.LOCAL_IP_PREFIX])

    @staticmethod
    @resource_extend.extends([ext_sg.SECURITYGROUPRULES])
    def _extend_security_group_rule_with_params(sg_rule_res, sg_rule_db):
        if sg_rule_db.ext_properties:
            sg_rule_res[ext_local_ip.LOCAL_IP_PREFIX] = (
                sg_rule_db.ext_properties.local_ip_prefix)
        else:
            sg_rule_res[ext_local_ip.LOCAL_IP_PREFIX] = None



# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from neutron.tests import base
from neutron_lib.plugins import constants

from vmware_nsx.plugins.nsx_v.housekeeper import error_backup_edge

FAKE_ROUTER_BINDINGS = [
    {
        'router_id': 'backup-3b0b1fe1-c984', 'status': 'ERROR',
        'availability_zone': 'default', 'edge_id': 'edge-782',
        'edge_type': 'service', 'appliance_size': 'compact'}]


class ErrorBackupEdgeTestCaseReadOnly(base.BaseTestCase):
    def _is_readonly(self):
        return True

    def setUp(self):
        def get_plugin_mock(alias=constants.CORE):
            if alias in (constants.CORE, constants.L3):
                return self.plugin

        super(ErrorBackupEdgeTestCaseReadOnly, self).setUp()
        self.plugin = mock.Mock()
        self.context = mock.Mock()
        self.context.session = mock.Mock()
        mock.patch('neutron_lib.plugins.directory.get_plugin',
                   side_effect=get_plugin_mock).start()
        self.log = mock.Mock()
        error_backup_edge.LOG = self.log
        self.job = error_backup_edge.ErrorBackupEdgeJob(self._is_readonly())

    def test_clean_run(self):
        mock.patch('vmware_nsx.db.nsxv_db.get_nsxv_router_bindings',
                   return_value=[]).start()
        self.job.run(self.context)
        self.log.warning.assert_not_called()

    def test_broken_backup_edge(self):
        mock.patch('vmware_nsx.db.nsxv_db.get_nsxv_router_bindings',
                   return_value=FAKE_ROUTER_BINDINGS).start()

        self.job.run(self.context)
        self.log.warning.assert_called_once()


class ErrorBackupEdgeTestCaseReadWrite(ErrorBackupEdgeTestCaseReadOnly):
    def _is_readonly(self):
        return False

    def test_broken_backup_edge(self):
        upd_binding = mock.patch(
            'vmware_nsx.db.nsxv_db.update_nsxv_router_binding').start()
        upd_edge = mock.patch.object(self.plugin.nsx_v, 'update_edge').start()
        self.job.azs = mock.Mock()
        az = mock.Mock()
        mock.patch.object(self.job.azs, 'get_availability_zone',
                          return_value=az).start()
        super(ErrorBackupEdgeTestCaseReadWrite, self
              ).test_broken_backup_edge()
        upd_binding.assert_has_calls(
            [mock.call(mock.ANY, r['router_id'], status='ACTIVE')
             for r in FAKE_ROUTER_BINDINGS])
        upd_edge.assert_called_with(
            self.context, 'backup-3b0b1fe1-c984', 'edge-782',
            'backup-3b0b1fe1-c984', None, appliance_size='compact',
            availability_zone=az, dist=False)



# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vmware_nsxlib.v3 import vpn_ipsec

ENCRYPTION_ALGORITHM_MAP = {
    'aes-128': vpn_ipsec.EncryptionAlgorithmTypes.ENCRYPTION_ALGORITHM_128,
    'aes-256': vpn_ipsec.EncryptionAlgorithmTypes.ENCRYPTION_ALGORITHM_256,
}

AUTH_ALGORITHM_MAP = {
    'sha1': vpn_ipsec.DigestAlgorithmTypes.DIGEST_ALGORITHM_SHA1,
    'sha256': vpn_ipsec.DigestAlgorithmTypes.DIGEST_ALGORITHM_SHA256,
}

PFS_MAP = {
    'group14': vpn_ipsec.DHGroupTypes.DH_GROUP_14
}

IKE_VERSION_MAP = {
    'v1': vpn_ipsec.IkeVersionTypes.IKE_VERSION_V1,
    'v2': vpn_ipsec.IkeVersionTypes.IKE_VERSION_V2,
}

ENCAPSULATION_MODE_MAP = {
    'tunnel': vpn_ipsec.EncapsulationModeTypes.ENCAPSULATION_MODE_TUNNEL
}

TRANSFORM_PROTOCOL_MAP = {
    'esp': vpn_ipsec.TransformProtocolTypes.TRANSFORM_PROTOCOL_ESP
}

DPD_ACTION_MAP = {
    'hold': vpn_ipsec.DpdProfileActionTypes.DPD_PROFILE_ACTION_HOLD,
    'disabled': None
}

INITIATION_MODE_MAP = {
    'bi-directional': (vpn_ipsec.ConnectionInitiationModeTypes.
        INITIATION_MODE_INITIATOR),
    'response-only': (vpn_ipsec.ConnectionInitiationModeTypes.
        INITIATION_MODE_RESPOND_ONLY)
}

DEFAULT_LOG_LEVEL = vpn_ipsec.IkeLogLevelTypes.LOG_LEVEL_ERROR



# Copyright (c) 2015 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import mock
from oslo_config import cfg

from neutron.tests.unit import testlib_api

from vmware_nsx.common import exceptions as nsx_exc
from vmware_nsx.plugins.nsx_v3 import cert_utils
from vmware_nsx.plugins.nsx_v3 import utils


class NsxV3ClientCertProviderTestCase(testlib_api.SqlTestCase):

    CERT = "-----BEGIN CERTIFICATE-----\n" \
        "MIIDJTCCAg0CBFh36j0wDQYJKoZIhvcNAQELBQAwVzELMAkGA1UEBhMCVVMxEzAR\n" \
        "BgNVBAgMCkNhbGlmb3JuaWExDjAMBgNVBAoMBU15T3JnMQ8wDQYDVQQLDAZNeVVu\n" \
        "aXQxEjAQBgNVBAMMCW15b3JnLmNvbTAeFw0xNzAxMTIyMDQyMzdaFw0yNzAxMTAy\n" \
        "MDQyMzdaMFcxCzAJBgNVBAYTAlVTMRMwEQYDVQQIDApDYWxpZm9ybmlhMQ4wDAYD\n" \
        "VQQKDAVNeU9yZzEPMA0GA1UECwwGTXlVbml0MRIwEAYDVQQDDAlteW9yZy5jb20w\n" \
        "ggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQC/wsYintlWVaSeXwaSrdPa\n" \
        "+AHtL1ooH7q0uf6tt+6Rwiy10YRjAVJhapj9995gqgJ2402J+3gzNXLCbXjjDR/D\n" \
        "9xjAzKHu61r0AVNd9/0+8yXQrEDuzlwHSCKz+zjq5ZEZ7RkLIUdreaZJFPTCwry3\n" \
        "wuTnBfqcE7xWl6WfWR8evooV+ZzIfjQdoSliIyn3YGxNN5pc1P40qt0pxOsNBGXG\n" \
        "2FIZXpML8TpKw0ga/wE70CJd6tRvSsAADxQXehfKvGtHvlJYS+3cTahC7reQXJnc\n" \
        "qsjgYkiWyhhR4jdcTD/tDlVcJroM1jFVxpsCg/AU3srWWWeAGyVe42ZhqWVf0Urz\n" \
        "AgMBAAEwDQYJKoZIhvcNAQELBQADggEBAA/lLfmXe8wPyBhN/VMb5bu5Ey56qz+j\n" \
        "jCn7tz7FjRvsB9P0fLUDOBKNwyon3yopDNYJ4hnm4yKoHCHURQLZKWHzm0XKzE+4\n" \
        "cA/M13M8OEg5otnVVHhz1FPQWnJq7bLHh/KXYcc5Rkc7UeHEPj0sDjfUjCPGdepc\n" \
        "Ghu1ZcgHsL4JCuvcadG+RFGeDTug3yO92Fj2uFy5DlzzWOZSi4otpZRd9JZkAtZ1\n" \
        "umZRBJ2A504nJx4MplmNqvLNkmxMLKQdvZYNNiYr6icOavDOJA5RhzgoppJZkV2w\n" \
        "v2oC+8BFarXnZSk37HAWjwcaqzBLbIyPYpClW5IYMr8LiixSBACc+4w=\n" \
        "-----END CERTIFICATE-----\n"

    PKEY = "-----BEGIN PRIVATE KEY-----\n" \
        "MIIEwAIBADANBgkqhkiG9w0BAQEFAASCBKowggSmAgEAAoIBAQC/wsYintlWVaSe\n" \
        "XwaSrdPa+AHtL1ooH7q0uf6tt+6Rwiy10YRjAVJhapj9995gqgJ2402J+3gzNXLC\n" \
        "bXjjDR/D9xjAzKHu61r0AVNd9/0+8yXQrEDuzlwHSCKz+zjq5ZEZ7RkLIUdreaZJ\n" \
        "FPTCwry3wuTnBfqcE7xWl6WfWR8evooV+ZzIfjQdoSliIyn3YGxNN5pc1P40qt0p\n" \
        "xOsNBGXG2FIZXpML8TpKw0ga/wE70CJd6tRvSsAADxQXehfKvGtHvlJYS+3cTahC\n" \
        "7reQXJncqsjgYkiWyhhR4jdcTD/tDlVcJroM1jFVxpsCg/AU3srWWWeAGyVe42Zh\n" \
        "qWVf0UrzAgMBAAECggEBAJrGuie9cQy3KZzOdD614RaPMPbhTnKuUYOH0GEk4YFy\n" \
        "aaYDS0iiC30njf8HLs10y3JsOuyRNU6X6F24AGe68xW3/pm3UUjHXG0wGLry68wA\n" \
        "c1g/gFV/6FXUSnZc4m7uBjUX4yvRm5TK5oV8TaZZifsEar9xWvrZDx4RXpQEWhL0\n" \
        "L/TyrOZSfRtBgdWX6Ag4XQVsCfZxJoCi2ZyvaMBsWTH06x9AGo1Io5t1AmA9Hsfb\n" \
        "6BsSz186nqb0fq4UMfrWrSCz7M/1s03+hBOVICH2TdaRDZLtDVa1b2x4sFpfdp9t\n" \
        "VVxuSHxcmvzOPMIv3NXwj0VitTYYJDBFKoEfx1mzhNkCgYEA59gYyBfpsuCOevP2\n" \
        "tn7IeysbtaoKDzHE+ksjs3sAn6Vr2Y0Lbed26NpdIVL6u3HAteJxqrIh0zpkpAtp\n" \
        "akdqlj86oRaBUqLXxK3QNpUx19f7KN7UsVAbzUJSOm2n1piPg261ktfhtms2rxnQ\n" \
        "+9yluINu+z1wS4FG9SwrRmwwfsUCgYEA072Ma1sj2MER5tmQw1zLANkzP1PAkUdy\n" \
        "+oDuJmU9A3/+YSIkm8dGprFglPkLUaf1B15oN6wCJVMpB1lza3PM/YT70rpqc7cq\n" \
        "PHJXQlZFMBhyVfIkCv3wICTLD5phhgAWlzlwm094f2uAnbG6WUkrVfZajuh0pW53\n" \
        "1i0OTfxAvlcCgYEAkDB2oSM2JhjApDlMbA2HtAqIbkA1h2OlpSDMMFjEd4WTALdW\n" \
        "r2CwNHtyRkJsS92gQ750gPvOS6daZifuxLlr0cu7M+piPbmnRdvvzbKWUC40NyP2\n" \
        "1dwDnnGr4EjIhI9XWh+lb5EyAJjHZrlAnxOIQawEft6kE2FwdxSkSWUJ+B0CgYEA\n" \
        "n2xYDXzRwKGdmPK2zGFRd5IRw9yLYNcq+vGYXdBb4Aa+wOO0LJYd2+Qxk/jvTMvo\n" \
        "8WNjlIcuFmxGuAHhpUXLUhaOhFtXS0jdxCVTDd9muI+vhoaKHLyVz53kRhs20m2+\n" \
        "lJ3q6wUq9MU8UX8/j3pH5rFV/cOIEAbcs6W4337OQIECgYEAoLtQyqXjH45FlCQx\n" \
        "xK8dY+GuxIP+TIwiq23yhu3e+3LIgXJw8DwBFN5yJyH2HMnhGkD4PurEx2sGHeLO\n" \
        "EG6L8PNDOxpvSzcgxwmZsUK6j3nAbKycF3PDDXA4kt8WDXBr86OMQsFtpjeO+fGh\n" \
        "YWJa+OKc2ExdeMewe9gKIDQ5stw=\n" \
        "-----END PRIVATE KEY-----\n"

    def _init_config(self, storage_type='nsx-db',
                     password=None, cert_file=None):
        cfg.CONF.set_override('nsx_use_client_auth', True, 'nsx_v3')
        cfg.CONF.set_override('nsx_client_cert_storage',
                              storage_type, 'nsx_v3')

        cfg.CONF.set_override('nsx_client_cert_file', cert_file, 'nsx_v3')
        cfg.CONF.set_override('nsx_client_cert_pk_password',
                              password, 'nsx_v3')

        # pk password secret is cached - reset it for each test
        cert_utils.reset_secret()

        self._provider = utils.get_client_cert_provider()

    def validate_db_provider(self, expected_cert_data):
        fname = None
        with self._provider() as p:
            # verify cert data was exported to CERTFILE
            fname = p.filename()
            with open(fname, 'r') as f:
                actual = f.read()

            self.assertEqual(expected_cert_data, actual)

        # after with statement, cert file should be deleted
        self.assertFalse(os.path.isfile(fname))

    def validate_basic_provider(self, expected_cert_data):
        fname = None
        with self._provider as p:
            fname = p.filename()
            with open(fname, 'r') as f:
                actual = f.read()

            self.assertEqual(expected_cert_data, actual)

        # with statement should not touch the file
        self.assertTrue(os.path.isfile(fname))

    def test_db_provider_without_cert(self):
        """Verify init fails if no cert is provided in client cert mode"""
        # certificate not generated - exception should be raised
        self._init_config()

        # no certificate in table
        mock.patch(
            "vmware_nsx.db.db.get_certificate",
            return_value=(None, None)).start()
        self.assertRaises(nsx_exc.ClientCertificateException,
                          self._provider().__enter__)

        # now verify return to normal after failure
        mock.patch(
            "vmware_nsx.db.db.get_certificate",
            return_value=(self.CERT, self.PKEY)).start()

        self.validate_db_provider(self.CERT + self.PKEY)

    def test_db_provider_with_cert(self):
        """Verify successful certificate load from storage"""

        self._init_config()

        mock.patch(
            "vmware_nsx.db.db.get_certificate",
            return_value=(self.CERT, self.PKEY)).start()

        self.validate_db_provider(self.CERT + self.PKEY)

    def test_db_provider_with_encryption(self):
        """Verify successful encrypted PK load from storage"""

        password = 'topsecret'
        self._init_config(password=password)
        secret = cert_utils.generate_secret_from_password(password)
        encrypted_pkey = cert_utils.symmetric_encrypt(secret, self.PKEY)

        # db should contain encrypted key
        mock.patch(
            "vmware_nsx.db.db.get_certificate",
            return_value=(self.CERT, encrypted_pkey)).start()

        self.validate_db_provider(self.CERT + self.PKEY)

    def test_db_provider_with_bad_decrypt(self):
        """Verify loading plaintext PK from storage fails in encrypt mode"""

        mock.patch(
            "vmware_nsx.db.db.get_certificate",
            return_value=(self.CERT, self.PKEY)).start()
        # after decrypt failure, cert will be deleted
        mock.patch(
            "vmware_nsx.db.db.delete_certificate").start()

        self._init_config(password='topsecret')

        # since PK in DB is not encrypted, we should fail to decrypt it on load
        self.assertRaises(nsx_exc.ClientCertificateException,
                          self._provider().__enter__)

    def test_basic_provider(self):

        fname = '/tmp/cert.pem'
        # with basic provider, the file is provided by admin
        with open(fname, 'w') as f:
            f.write(self.CERT)
            f.write(self.PKEY)

        self._init_config(storage_type='none', cert_file=fname)
        with self._provider as p:
            self.assertEqual(fname, p.filename())

        self.validate_basic_provider(self.CERT + self.PKEY)
        os.remove(fname)



# Copyright 2014 VMware, Inc.
#
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import sys

from neutronclient import shell
from oslo_config import cfg
from oslo_log import log as logging

from vmware_nsx.shell import commands as cmd

# Oslo Logging uses INFO as default
# Use a simple format for the output
logging_format_string = '%(message)s'
logging.register_options(cfg.CONF)
logging.setup(cfg.CONF, "vmware-nsx")
cfg.CONF.set_override('logging_context_format_string',
                      logging_format_string)
cfg.CONF.set_override('logging_default_format_string',
                      logging_format_string)
cfg.CONF.set_override('logging_exception_prefix',
                      '')


class NsxManage(shell.NeutronShell):

    def __init__(self, api_version):
        super(NsxManage, self).__init__(api_version)
        self.command_manager.add_command('net-migrate', cmd.NetworkMigrate)
        self.command_manager.add_command('net-report', cmd.NetworkReport)

    def build_option_parser(self, description, version):
        parser = super(NsxManage, self).build_option_parser(
            description, version)
        return parser

    def initialize_app(self, argv):
        super(NsxManage, self).initialize_app(argv)
        self.client = self.client_manager.neutron


def main():
    return NsxManage(shell.NEUTRON_API_VERSION).run(sys.argv[1:])
